@@ -76,6 +76,9 @@ enum vkd3d_shader_meta_flags
     VKD3D_SHADER_META_FLAG_EMITS_TRIANGLES = 1 << 15,
     VKD3D_SHADER_META_FLAG_FORCE_COMPUTE_BARRIER_AFTER_DISPATCH = 1 << 16,
     VKD3D_SHADER_META_FLAG_EXPORTS_SAMPLE_MASK = 1 << 17,
+    VKD3D_SHADER_META_FLAG_FORCE_PRE_RASTERIZATION_BEFORE_DISPATCH = 1 << 18,
+    VKD3D_SHADER_META_FLAG_FORCE_GRAPHICS_BEFORE_DISPATCH = 1 << 19,
+    VKD3D_SHADER_META_FLAG_USES_DEPTH_STENCIL_WRITE = 1 << 20,
 };
 
 struct vkd3d_shader_meta
@@ -83,7 +86,10 @@ struct vkd3d_shader_meta
     vkd3d_shader_hash_t hash;
     unsigned int cs_workgroup_size[3]; /* Only contains valid data if uses_subgroup_size is true. */
     unsigned int patch_vertex_count; /* Relevant for HS. May be 0, in which case the patch vertex count is not known. */
-    unsigned int cs_required_wave_size; /* If non-zero, force a specific CS subgroup size. */
+    uint8_t cs_wave_size_min; /* If non-zero, minimum or required subgroup size. */
+    uint8_t cs_wave_size_max; /* If non-zero, maximum subgroup size. */
+    uint8_t cs_wave_size_preferred; /* If non-zero, preferred subgroup size. */
+    uint8_t reserved;
     uint32_t flags; /* vkd3d_shader_meta_flags */
 };
 STATIC_ASSERT(sizeof(struct vkd3d_shader_meta) == 32);
@@ -430,6 +436,11 @@ enum vkd3d_shader_quirk
 
     /* Range check every descriptor heap access with dynamic index and robustness check it. */
     VKD3D_SHADER_QUIRK_DESCRIPTOR_HEAP_ROBUSTNESS = (1 << 15),
+
+    /* Requests META_FLAG_FORCE_PRE_RASTERIZATION_BEFORE_DISPATCH to be set in shader meta. */
+    VKD3D_SHADER_QUIRK_FORCE_PRE_RASTERIZATION_BARRIER = (1 << 16),
+    /* Requests META_FLAG_FORCE_GRAPHICS_BEFORE_DISPATCH to be set in shader meta. */
+    VKD3D_SHADER_QUIRK_FORCE_GRAPHICS_BARRIER = (1 << 17),
 };
 
 struct vkd3d_shader_quirk_hash
@@ -468,6 +479,9 @@ struct vkd3d_shader_compile_arguments
     bool promote_wave_size_heuristics;
 
     const struct vkd3d_shader_quirk_info *quirks;
+    /* Only non-zero when enabled by vkd3d_config */
+    VkDriverId driver_id;
+    uint32_t driver_version;
 };
 
 enum vkd3d_tessellator_output_primitive
