@@ -558,6 +558,7 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
         const struct vkd3d_shader_compile_arguments *compiler_args)
 {
     dxil_spv_option_denorm_preserve_support denorm_preserve = {{ DXIL_SPV_OPTION_DENORM_PRESERVE_SUPPORT }};
+    uint32_t wave_size_min, wave_size_max, wave_size_preferred;
     struct vkd3d_dxil_remap_userdata remap_userdata;
     bool compute_shader_derivatives = false;
     unsigned int raw_va_binding_count = 0;
@@ -590,6 +591,10 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
     quirks = vkd3d_shader_compile_arguments_select_quirks(compiler_args, hash);
     if (quirks & VKD3D_SHADER_QUIRK_FORCE_COMPUTE_BARRIER)
         spirv->meta.flags |= VKD3D_SHADER_META_FLAG_FORCE_COMPUTE_BARRIER_AFTER_DISPATCH;
+    if (quirks & VKD3D_SHADER_QUIRK_FORCE_PRE_RASTERIZATION_BARRIER)
+        spirv->meta.flags |= VKD3D_SHADER_META_FLAG_FORCE_PRE_RASTERIZATION_BEFORE_DISPATCH;
+    if (quirks & VKD3D_SHADER_QUIRK_FORCE_GRAPHICS_BARRIER)
+        spirv->meta.flags |= VKD3D_SHADER_META_FLAG_FORCE_GRAPHICS_BEFORE_DISPATCH;
 
     dxil_spv_begin_thread_allocator_context();
 
@@ -983,6 +988,20 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
             }
         }
 
+        if (compiler_args->driver_version)
+        {
+            const dxil_spv_option_driver_version version = {
+                    { DXIL_SPV_OPTION_DRIVER_VERSION },
+                    compiler_args->driver_id, compiler_args->driver_version };
+
+            if (dxil_spv_converter_add_option(converter, &version.base) != DXIL_SPV_SUCCESS)
+            {
+                ERR("dxil-spirv does not support DRIVER_VERSION.\n");
+                ret = VKD3D_ERROR_NOT_IMPLEMENTED;
+                goto end;
+            }
+        }
+
         for (i = 0; i < compiler_args->parameter_count; i++)
         {
             const struct vkd3d_shader_parameter *argument = &compiler_args->parameters[i];
@@ -1118,7 +1137,13 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
             &spirv->meta.cs_workgroup_size[1],
             &spirv->meta.cs_workgroup_size[2]);
     dxil_spv_converter_get_patch_vertex_count(converter, &spirv->meta.patch_vertex_count);
-    dxil_spv_converter_get_compute_required_wave_size(converter, &spirv->meta.cs_required_wave_size);
+
+    dxil_spv_converter_get_compute_wave_size_range(converter,
+            &wave_size_min, &wave_size_max, &wave_size_preferred);
+
+    /* Ensure that the maximum wave size is always valid */
+    if (!wave_size_max)
+        wave_size_max = wave_size_min;
 
     if (compiler_args->promote_wave_size_heuristics)
     {
@@ -1126,12 +1151,16 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
         if (quirks & VKD3D_SHADER_QUIRK_FORCE_MAX_WAVE32)
             heuristic_wave_size = 32;
 
-        if (heuristic_wave_size && !spirv->meta.cs_required_wave_size &&
+        if (heuristic_wave_size && !wave_size_min &&
                 compiler_args->max_subgroup_size > heuristic_wave_size &&
                 compiler_args->min_subgroup_size <= heuristic_wave_size)
-            spirv->meta.cs_required_wave_size = heuristic_wave_size;
+            wave_size_preferred = heuristic_wave_size;
     }
 
+    spirv->meta.cs_wave_size_min = wave_size_min;
+    spirv->meta.cs_wave_size_max = wave_size_max;
+    spirv->meta.cs_wave_size_preferred = wave_size_preferred;
+
     vkd3d_shader_extract_feature_meta(spirv);
     vkd3d_shader_dump_spirv_shader(hash, spirv);
 
@@ -1174,6 +1203,10 @@ int vkd3d_shader_compile_dxil_export(const struct vkd3d_shader_code *dxil,
     quirks = vkd3d_shader_compile_arguments_select_quirks(compiler_args, hash);
     if (quirks & VKD3D_SHADER_QUIRK_FORCE_COMPUTE_BARRIER)
         spirv->meta.flags |= VKD3D_SHADER_META_FLAG_FORCE_COMPUTE_BARRIER_AFTER_DISPATCH;
+    if (quirks & VKD3D_SHADER_QUIRK_FORCE_PRE_RASTERIZATION_BARRIER)
+        spirv->meta.flags |= VKD3D_SHADER_META_FLAG_FORCE_PRE_RASTERIZATION_BEFORE_DISPATCH;
+    if (quirks & VKD3D_SHADER_QUIRK_FORCE_GRAPHICS_BARRIER)
+        spirv->meta.flags |= VKD3D_SHADER_META_FLAG_FORCE_GRAPHICS_BEFORE_DISPATCH;
 
     /* For user provided (not mangled) export names, just inherit that name. */
     if (!demangled_export)
@@ -1608,6 +1641,20 @@ int vkd3d_shader_compile_dxil_export(const struct vkd3d_shader_code *dxil,
                 }
             }
         }
+
+        if (compiler_args->driver_version)
+        {
+            const dxil_spv_option_driver_version version = {
+                    { DXIL_SPV_OPTION_DRIVER_VERSION },
+                    compiler_args->driver_id, compiler_args->driver_version };
+
+            if (dxil_spv_converter_add_option(converter, &version.base) != DXIL_SPV_SUCCESS)
+            {
+                ERR("dxil-spirv does not support DRIVER_VERSION.\n");
+                ret = VKD3D_ERROR_NOT_IMPLEMENTED;
+                goto end;
+            }
+        }
     }
 
     if (dxil_spv_converter_add_option(converter, &denorm_preserve.base) != DXIL_SPV_SUCCESS)
