@@ -155,7 +155,7 @@ HRESULT vkd3d_breadcrumb_tracer_init(struct vkd3d_breadcrumb_tracer *tracer, str
         resource_desc.Flags = D3D12_RESOURCE_FLAG_NONE;
 
         if (FAILED(hr = vkd3d_create_buffer(device, &heap_properties, D3D12_HEAP_FLAG_ALLOW_ONLY_BUFFERS,
-                &resource_desc, &tracer->host_buffer)))
+                &resource_desc, "breadcrumb-host-buffer", &tracer->host_buffer)))
         {
             goto err;
         }
@@ -838,10 +838,12 @@ void vkd3d_breadcrumb_tracer_end_command_list(struct d3d12_command_list *list)
 
 void vkd3d_breadcrumb_tracer_update_barrier_hashes(struct vkd3d_breadcrumb_tracer *tracer)
 {
+    vkd3d_shader_hash_t lo_hash;
+    vkd3d_shader_hash_t hi_hash;
     char env[VKD3D_PATH_MAX];
-    vkd3d_shader_hash_t hash;
     uint32_t new_count;
-    char line[17];
+    char line[64];
+    char *end_ptr;
     FILE *file;
 
     if (vkd3d_get_env_var("VKD3D_BARRIER_HASHES", env, sizeof(env)))
@@ -853,10 +855,29 @@ void vkd3d_breadcrumb_tracer_update_barrier_hashes(struct vkd3d_breadcrumb_trace
             new_count = 0;
             while (fgets(line, sizeof(line), file))
             {
-                hash = strtoull(line, NULL, 16);
-                vkd3d_array_reserve((void **)&tracer->barrier_hashes, &tracer->barrier_hashes_size,
-                        new_count + 1, sizeof(*tracer->barrier_hashes));
-                tracer->barrier_hashes[new_count++] = hash;
+                /* Look for either a single number, or lohash-hihash format. */
+                if (!isalnum(*line))
+                    continue;
+
+                lo_hash = strtoull(line, &end_ptr, 16);
+
+                while (*end_ptr != '\0' && !isalnum(*end_ptr))
+                    end_ptr++;
+
+                hi_hash = strtoull(end_ptr, NULL, 16);
+
+                if (!hi_hash)
+                    hi_hash = lo_hash;
+
+                if (lo_hash || hi_hash)
+                {
+                    vkd3d_array_reserve((void **)&tracer->barrier_hashes, &tracer->barrier_hashes_size,
+                            new_count + 1, sizeof(*tracer->barrier_hashes));
+
+                    tracer->barrier_hashes[new_count].lo = lo_hash;
+                    tracer->barrier_hashes[new_count].hi = hi_hash;
+                    new_count++;
+                }
             }
             vkd3d_atomic_uint32_store_explicit(&tracer->barrier_hashes_count, new_count, vkd3d_memory_order_relaxed);
             pthread_mutex_unlock(&tracer->barrier_hash_lock);
@@ -878,7 +899,7 @@ bool vkd3d_breadcrumb_tracer_shader_hash_forces_barrier(struct vkd3d_breadcrumb_
     {
         pthread_mutex_lock(&tracer->barrier_hash_lock);
         for (i = 0; i < tracer->barrier_hashes_count && !ret; i++)
-            ret = tracer->barrier_hashes[i] == hash;
+            ret = tracer->barrier_hashes[i].lo <= hash && hash <= tracer->barrier_hashes[i].hi;
         pthread_mutex_unlock(&tracer->barrier_hash_lock);
     }
     return ret;
