@@ -51,7 +51,7 @@ static HRESULT STDMETHODCALLTYPE d3d12_device_vkd3d_ext_QueryInterface(d3d12_dev
 static HRESULT STDMETHODCALLTYPE d3d12_device_vkd3d_ext_GetVulkanHandles(d3d12_device_vkd3d_ext_iface *iface, VkInstance *vk_instance, VkPhysicalDevice *vk_physical_device, VkDevice *vk_device)
 {
     struct d3d12_device *device = d3d12_device_from_ID3D12DeviceExt(iface);
-    TRACE("iface %p, vk_instance %p, vk_physical_device %p, vk_device %p \n", iface, vk_instance, vk_physical_device, vk_device);
+    TRACE("iface %p, vk_instance %p, vk_physical_device %p, vk_device %p\n", iface, vk_instance, vk_physical_device, vk_device);
     if (!vk_device || !vk_instance || !vk_physical_device)
         return E_INVALIDARG;
         
@@ -66,7 +66,7 @@ static BOOL STDMETHODCALLTYPE d3d12_device_vkd3d_ext_GetExtensionSupport(d3d12_d
     const struct d3d12_device *device = d3d12_device_from_ID3D12DeviceExt(iface);
     bool ret_val = false;
     
-    TRACE("iface %p, extension %u \n", iface, extension);
+    TRACE("iface %p, extension %u\n", iface, extension);
     switch (extension)
     {
         case D3D12_VK_NVX_BINARY_IMPORT:
@@ -79,7 +79,7 @@ static BOOL STDMETHODCALLTYPE d3d12_device_vkd3d_ext_GetExtensionSupport(d3d12_d
             ret_val = device->vk_info.NV_low_latency2;
             break;
         default:
-            WARN("Invalid extension %x\n", extension);
+            WARN("Invalid extension %x.\n", extension);
     }
     
     return ret_val;
@@ -96,7 +96,7 @@ static HRESULT STDMETHODCALLTYPE d3d12_device_vkd3d_ext_CreateCubinComputeShader
     VkDevice vk_device;
     VkResult vr;
     
-    TRACE("iface %p, cubin_data %p, cubin_size %u, shader_name %s \n", iface, cubin_data, cubin_size, shader_name);
+    TRACE("iface %p, cubin_data %p, cubin_size %u, shader_name %s\n", iface, cubin_data, cubin_size, shader_name);
     if (!cubin_data || !cubin_size || !shader_name)
         return E_INVALIDARG;    
 
@@ -138,7 +138,7 @@ static HRESULT STDMETHODCALLTYPE d3d12_device_vkd3d_ext_DestroyCubinComputeShade
     struct d3d12_device *device;
     VkDevice vk_device;
 
-    TRACE("iface %p, handle %p \n", iface, handle);
+    TRACE("iface %p, handle %p\n", iface, handle);
     if (!iface || !handle)
         return E_INVALIDARG;
     
@@ -218,7 +218,36 @@ static HRESULT STDMETHODCALLTYPE d3d12_device_vkd3d_ext_CaptureUAVInfo(d3d12_dev
     return S_OK;
 }
 
-CONST_VTBL struct ID3D12DeviceExtVtbl d3d12_device_vkd3d_ext_vtbl =
+static HRESULT STDMETHODCALLTYPE d3d12_device_vkd3d_ext_CreateResourceFromBorrowedHandle(d3d12_device_vkd3d_ext_iface *iface,
+       const D3D12_RESOURCE_DESC1 *desc, UINT64 vk_handle, ID3D12Resource **ppResource)
+{
+    struct d3d12_device *device = d3d12_device_from_ID3D12DeviceExt(iface);
+    struct d3d12_resource *object;
+    HRESULT hr = d3d12_resource_create_borrowed(device, desc, vk_handle, &object);
+    if (FAILED(hr))
+        return hr;
+
+    *ppResource = (ID3D12Resource *)&object->ID3D12Resource_iface;
+    return S_OK;
+}
+
+static HRESULT STDMETHODCALLTYPE d3d12_device_vkd3d_ext_GetVulkanQueueInfoEx(d3d12_device_vkd3d_ext_iface *iface,
+       ID3D12CommandQueue *queue, VkQueue *vk_queue, UINT32 *vk_queue_index, UINT32 *vk_queue_flags, UINT32 *vk_queue_family)
+{
+    TRACE("iface %p, queue %p, vk_queue %p, vk_queue_index %p, vk_queue_flags %p vk_queue_family %p.\n",
+            iface, queue, vk_queue, vk_queue_index, vk_queue_flags, vk_queue_family);
+
+    /* This only gets called during D3D11 device creation */
+    *vk_queue = vkd3d_acquire_vk_queue(queue);
+    vkd3d_release_vk_queue(queue);
+
+    *vk_queue_index = vkd3d_get_vk_queue_index(queue);
+    *vk_queue_flags = vkd3d_get_vk_queue_flags(queue);
+    *vk_queue_family = vkd3d_get_vk_queue_family_index(queue);
+    return S_OK;
+}
+
+CONST_VTBL struct ID3D12DeviceExt1Vtbl d3d12_device_vkd3d_ext_vtbl =
 {
     /* IUnknown methods */
     d3d12_device_vkd3d_ext_QueryInterface,
@@ -232,7 +261,11 @@ CONST_VTBL struct ID3D12DeviceExtVtbl d3d12_device_vkd3d_ext_vtbl =
     d3d12_device_vkd3d_ext_DestroyCubinComputeShader,
     d3d12_device_vkd3d_ext_GetCudaTextureObject,
     d3d12_device_vkd3d_ext_GetCudaSurfaceObject,
-    d3d12_device_vkd3d_ext_CaptureUAVInfo
+    d3d12_device_vkd3d_ext_CaptureUAVInfo,
+
+    /* ID3D12DeviceExt1 methods */
+    d3d12_device_vkd3d_ext_CreateResourceFromBorrowedHandle,
+    d3d12_device_vkd3d_ext_GetVulkanQueueInfoEx,
 };
 
 
@@ -276,7 +309,7 @@ static HRESULT STDMETHODCALLTYPE d3d12_dxvk_interop_device_GetVulkanHandles(ID3D
         VkInstance *vk_instance, VkPhysicalDevice *vk_physical_device, VkDevice *vk_device)
 {
     struct d3d12_device *device = d3d12_device_from_ID3D12DXVKInteropDevice(iface);
-    TRACE("iface %p, vk_instance %p, vk_physical_device %p, vk_device %p \n", iface, vk_instance, vk_physical_device, vk_device);
+    TRACE("iface %p, vk_instance %p, vk_physical_device %p, vk_device %p\n", iface, vk_instance, vk_physical_device, vk_device);
     if (!vk_device || !vk_instance || !vk_physical_device)
         return E_INVALIDARG;
 
@@ -291,9 +324,9 @@ static HRESULT STDMETHODCALLTYPE d3d12_dxvk_interop_device_GetInstanceExtensions
     struct d3d12_device *device = d3d12_device_from_ID3D12DXVKInteropDevice(iface);
     struct vkd3d_instance *instance = device->vkd3d_instance;
 
-    TRACE("iface %p, extension_count %u, extensions %p.\n", iface, extension_count, extensions);
+    TRACE("iface %p, extension_count %p, extensions %p.\n", iface, extension_count, extensions);
 
-    if (extensions && (*extension_count < instance->vk_info.extension_count))
+    if (!extension_count || (extensions && (*extension_count < instance->vk_info.extension_count)))
         return E_INVALIDARG;
 
     *extension_count = instance->vk_info.extension_count;
@@ -310,9 +343,9 @@ static HRESULT STDMETHODCALLTYPE d3d12_dxvk_interop_device_GetDeviceExtensions(I
 {
     struct d3d12_device *device = d3d12_device_from_ID3D12DXVKInteropDevice(iface);
 
-    TRACE("iface %p, extension_count %u, extensions %p.\n", iface, extension_count, extensions);
+    TRACE("iface %p, extension_count %p, extensions %p.\n", iface, extension_count, extensions);
 
-    if (extensions && (*extension_count < device->vk_info.extension_count))
+    if (!extension_count || (extensions && (*extension_count < device->vk_info.extension_count)))
         return E_INVALIDARG;
 
     *extension_count = device->vk_info.extension_count;
@@ -524,19 +557,50 @@ static HRESULT STDMETHODCALLTYPE d3d12_low_latency_device_SetLatencyMarker(d3d_l
     if (!device->vk_info.NV_low_latency2)
         return E_NOTIMPL;
 
-    /* Offset the frameID by one to ensure it will always
-     * be a valid presentID */
-    internal_frame_id = frameID + 1;
+    if (frameID == 0)
+    {
+        WARN("FrameID is 0. Not a valid present ID.\n");
+        return S_OK;
+    }
+    else if (frameID >= (UINT64_MAX >> 1) / VKD3D_LOW_LATENCY_FRAME_ID_STRIDE)
+    {
+        /* Don't allow the frame ID to be set to the upper half of present ID space.
+         * We risk that swapchain runs out of IDs to increment if we allow application to set a present ID
+         * that is close enough to UINT64_MAX. */
+        WARN("FrameID %"PRIu64" is unexpectedly large. Effective present ID risks overflow. Ignoring.\n", frameID);
+        return S_OK;
+    }
+
+    /* Skip ahead. If application does not set frame counter, we'll still internally increment over time to fill in the gap.
+     * If application starts to use the frame IDs appropriately again, we'll catch up almost instantly,
+     * where low_latency_frame_id should overtake internal present ID counter.
+     * Frame marker needs to be device level monotonic. */
+    internal_frame_id = frameID * VKD3D_LOW_LATENCY_FRAME_ID_STRIDE;
 
     switch (vk_marker)
     {
         case VK_LATENCY_MARKER_SIMULATION_START_NV:
+            if (internal_frame_id <= device->frame_markers.simulation)
+            {
+                WARN("SIMULATION_START_NV is non-monotonic %"PRIu64" <= %"PRIu64".\n",
+                        internal_frame_id, device->frame_markers.simulation);
+            }
             device->frame_markers.simulation = internal_frame_id;
             break;
         case VK_LATENCY_MARKER_RENDERSUBMIT_START_NV:
+            if (internal_frame_id <= device->frame_markers.render)
+            {
+                WARN("RENDERSUBMIT_START_NV is non-monotonic %"PRIu64" <= %"PRIu64".\n",
+                        internal_frame_id, device->frame_markers.render);
+            }
             device->frame_markers.render = internal_frame_id;
             break;
         case VK_LATENCY_MARKER_PRESENT_START_NV:
+            if (internal_frame_id <= device->frame_markers.present)
+            {
+                WARN("PRESENT_START_NV is non-monotonic %"PRIu64" <= %"PRIu64".\n",
+                        internal_frame_id, device->frame_markers.present);
+            }
             device->frame_markers.present = internal_frame_id;
             break;
         default:
