@@ -21,7 +21,7 @@
 #define INITGUID
 #include "vkd3d_private.h"
 
-VKD3D_EXPORT HRESULT vkd3d_create_device(const struct vkd3d_device_create_info *create_info,
+HRESULT vkd3d_create_device(const struct vkd3d_device_create_info *create_info,
         REFIID iid, void **device)
 {
     struct vkd3d_instance *instance;
@@ -67,7 +67,7 @@ VKD3D_EXPORT HRESULT vkd3d_create_device(const struct vkd3d_device_create_info *
 
     if (!device)
     {
-        ID3D12Device_Release(&object->ID3D12Device_iface);
+        ID3D12Device12_Release(&object->ID3D12Device_iface);
         return S_FALSE;
     }
 
@@ -163,51 +163,6 @@ static CONST_VTBL struct ID3D12RootSignatureDeserializerVtbl d3d12_root_signatur
     d3d12_root_signature_deserializer_GetRootSignatureDesc,
 };
 
-static int vkd3d_parse_root_signature_for_version(const struct vkd3d_shader_code *dxbc,
-        struct vkd3d_versioned_root_signature_desc *out_desc,
-        enum vkd3d_root_signature_version target_version)
-{
-    struct vkd3d_versioned_root_signature_desc desc, converted_desc;
-    int ret;
-
-    if ((ret = vkd3d_shader_parse_root_signature(dxbc, &desc)) < 0)
-    {
-        WARN("Failed to parse root signature, vkd3d result %d.\n", ret);
-        return ret;
-    }
-
-    if (desc.version == target_version)
-    {
-        *out_desc = desc;
-    }
-    else
-    {
-        ret = vkd3d_shader_convert_root_signature(&converted_desc, target_version, &desc);
-        vkd3d_shader_free_root_signature(&desc);
-        if (ret < 0)
-        {
-            WARN("Failed to convert from version %#x, vkd3d result %d.\n", desc.version, ret);
-            return ret;
-        }
-
-        *out_desc = converted_desc;
-    }
-
-    return ret;
-}
-
-int vkd3d_parse_root_signature_v_1_0(const struct vkd3d_shader_code *dxbc,
-        struct vkd3d_versioned_root_signature_desc *out_desc)
-{
-    return vkd3d_parse_root_signature_for_version(dxbc, out_desc, VKD3D_ROOT_SIGNATURE_VERSION_1_0);
-}
-
-int vkd3d_parse_root_signature_v_1_1(const struct vkd3d_shader_code *dxbc,
-        struct vkd3d_versioned_root_signature_desc *out_desc)
-{
-    return vkd3d_parse_root_signature_for_version(dxbc, out_desc, VKD3D_ROOT_SIGNATURE_VERSION_1_1);
-}
-
 static HRESULT d3d12_root_signature_deserializer_init(struct d3d12_root_signature_deserializer *deserializer,
         const struct vkd3d_shader_code *dxbc)
 {
@@ -216,13 +171,13 @@ static HRESULT d3d12_root_signature_deserializer_init(struct d3d12_root_signatur
     deserializer->ID3D12RootSignatureDeserializer_iface.lpVtbl = &d3d12_root_signature_deserializer_vtbl;
     deserializer->refcount = 1;
 
-    if ((ret = vkd3d_parse_root_signature_v_1_0(dxbc, &deserializer->desc.vkd3d)) < 0)
+    if ((ret = vkd3d_shader_parse_root_signature_v_1_0(dxbc, &deserializer->desc.vkd3d, NULL)) < 0)
         return hresult_from_vkd3d_result(ret);
 
     return S_OK;
 }
 
-VKD3D_EXPORT HRESULT vkd3d_create_root_signature_deserializer(const void *data, SIZE_T data_size,
+HRESULT vkd3d_create_root_signature_deserializer(const void *data, SIZE_T data_size,
         REFIID iid, void **deserializer)
 {
     struct vkd3d_shader_code dxbc = {data, data_size};
@@ -255,7 +210,7 @@ struct d3d12_versioned_root_signature_deserializer
     {
         D3D12_VERSIONED_ROOT_SIGNATURE_DESC d3d12;
         struct vkd3d_versioned_root_signature_desc vkd3d;
-    } desc, other_desc;
+    } desc, other_desc[VKD3D_ROOT_SIGNATURE_VERSION_COUNT];
 };
 
 STATIC_ASSERT(sizeof(D3D12_VERSIONED_ROOT_SIGNATURE_DESC) == sizeof(struct vkd3d_versioned_root_signature_desc));
@@ -302,13 +257,15 @@ static ULONG STDMETHODCALLTYPE d3d12_versioned_root_signature_deserializer_Relea
 {
     struct d3d12_versioned_root_signature_deserializer *deserializer = impl_from_ID3D12VersionedRootSignatureDeserializer(iface);
     ULONG refcount = InterlockedDecrement(&deserializer->refcount);
+    unsigned int i;
 
     TRACE("%p decreasing refcount to %u.\n", deserializer, refcount);
 
     if (!refcount)
     {
         vkd3d_shader_free_root_signature(&deserializer->desc.vkd3d);
-        vkd3d_shader_free_root_signature(&deserializer->other_desc.vkd3d);
+        for (i = 0; i < ARRAY_SIZE(deserializer->other_desc); i++)
+            vkd3d_shader_free_root_signature(&deserializer->other_desc[i].vkd3d);
         vkd3d_free(deserializer);
     }
 
@@ -317,16 +274,7 @@ static ULONG STDMETHODCALLTYPE d3d12_versioned_root_signature_deserializer_Relea
 
 static enum vkd3d_root_signature_version vkd3d_root_signature_version_from_d3d12(D3D_ROOT_SIGNATURE_VERSION version)
 {
-    switch (version)
-    {
-        case D3D_ROOT_SIGNATURE_VERSION_1_0:
-            return VKD3D_ROOT_SIGNATURE_VERSION_1_0;
-        case D3D_ROOT_SIGNATURE_VERSION_1_1:
-            return VKD3D_ROOT_SIGNATURE_VERSION_1_1;
-        default:
-            WARN("Unknown root signature version %#x.\n", version);
-            return 0;
-    }
+    return (enum vkd3d_root_signature_version)version;
 }
 
 static HRESULT STDMETHODCALLTYPE d3d12_versioned_root_signature_deserializer_GetRootSignatureDescAtVersion(
@@ -334,11 +282,14 @@ static HRESULT STDMETHODCALLTYPE d3d12_versioned_root_signature_deserializer_Get
         const D3D12_VERSIONED_ROOT_SIGNATURE_DESC **desc)
 {
     struct d3d12_versioned_root_signature_deserializer *deserializer = impl_from_ID3D12VersionedRootSignatureDeserializer(iface);
+    enum vkd3d_root_signature_version vkd3d_root_sig_version;
+    unsigned int other_index;
     int ret;
 
     TRACE("iface %p, version %#x, desc %p.\n", iface, version, desc);
 
-    if (version != D3D_ROOT_SIGNATURE_VERSION_1_0 && version != D3D_ROOT_SIGNATURE_VERSION_1_1)
+    vkd3d_root_sig_version = vkd3d_root_signature_version_from_d3d12(version);
+    if (!vkd3d_root_signature_version_is_supported(vkd3d_root_sig_version))
     {
         WARN("Root signature version %#x not supported.\n", version);
         return E_INVALIDARG;
@@ -350,18 +301,20 @@ static HRESULT STDMETHODCALLTYPE d3d12_versioned_root_signature_deserializer_Get
         return S_OK;
     }
 
-    if (!deserializer->other_desc.d3d12.Version)
+    other_index = vkd3d_root_signature_version_to_other_index(vkd3d_root_sig_version);
+
+    if (!deserializer->other_desc[other_index].d3d12.Version)
     {
-        if ((ret = vkd3d_shader_convert_root_signature(&deserializer->other_desc.vkd3d,
-                vkd3d_root_signature_version_from_d3d12(version), &deserializer->desc.vkd3d)) < 0)
+        if ((ret = vkd3d_shader_convert_root_signature(&deserializer->other_desc[other_index].vkd3d,
+                vkd3d_root_sig_version, &deserializer->desc.vkd3d)) < 0)
         {
             WARN("Failed to convert versioned root signature, vkd3d result %d.\n", ret);
             return hresult_from_vkd3d_result(ret);
         }
     }
 
-    assert(deserializer->other_desc.d3d12.Version == version);
-    *desc = &deserializer->other_desc.d3d12;
+    assert(deserializer->other_desc[other_index].d3d12.Version == version);
+    *desc = &deserializer->other_desc[other_index].d3d12;
     return S_OK;
 }
 
@@ -394,7 +347,7 @@ static HRESULT d3d12_versioned_root_signature_deserializer_init(struct d3d12_ver
     deserializer->ID3D12VersionedRootSignatureDeserializer_iface.lpVtbl = &d3d12_versioned_root_signature_deserializer_vtbl;
     deserializer->refcount = 1;
 
-    if ((ret = vkd3d_shader_parse_root_signature(dxbc, &deserializer->desc.vkd3d)) < 0)
+    if ((ret = vkd3d_shader_parse_root_signature(dxbc, &deserializer->desc.vkd3d, NULL)) < 0)
     {
         WARN("Failed to parse root signature, vkd3d result %d.\n", ret);
         return hresult_from_vkd3d_result(ret);
@@ -405,7 +358,7 @@ static HRESULT d3d12_versioned_root_signature_deserializer_init(struct d3d12_ver
     return S_OK;
 }
 
-VKD3D_EXPORT HRESULT vkd3d_create_versioned_root_signature_deserializer(const void *data, SIZE_T data_size,
+HRESULT vkd3d_create_versioned_root_signature_deserializer(const void *data, SIZE_T data_size,
         REFIID iid, void **deserializer)
 {
     struct d3d12_versioned_root_signature_deserializer *object;
@@ -428,7 +381,7 @@ VKD3D_EXPORT HRESULT vkd3d_create_versioned_root_signature_deserializer(const vo
             &IID_ID3D12VersionedRootSignatureDeserializer, iid, deserializer);
 }
 
-VKD3D_EXPORT HRESULT vkd3d_serialize_root_signature(const D3D12_ROOT_SIGNATURE_DESC *desc,
+HRESULT vkd3d_serialize_root_signature(const D3D12_ROOT_SIGNATURE_DESC *desc,
         D3D_ROOT_SIGNATURE_VERSION version, ID3DBlob **blob, ID3DBlob **error_blob)
 {
     struct vkd3d_versioned_root_signature_desc vkd3d_desc;
@@ -476,7 +429,7 @@ VKD3D_EXPORT HRESULT vkd3d_serialize_root_signature(const D3D12_ROOT_SIGNATURE_D
     return S_OK;
 }
 
-VKD3D_EXPORT HRESULT vkd3d_serialize_versioned_root_signature(const D3D12_VERSIONED_ROOT_SIGNATURE_DESC *desc,
+HRESULT vkd3d_serialize_versioned_root_signature(const D3D12_VERSIONED_ROOT_SIGNATURE_DESC *desc,
         ID3DBlob **blob, ID3DBlob **error_blob)
 {
     const struct vkd3d_versioned_root_signature_desc *vkd3d_desc;
