@@ -63,52 +63,9 @@ void test_copy_texture(void)
         0x00000000, 0x00000000, 0x00000000, 0x00000000,
     };
     static const float white[] = {1.0f, 1.0f, 1.0f, 1.0f};
-    static const DWORD ps_code[] =
-    {
-#if 0
-        Texture2D<float> t;
 
-        float main(float4 position : SV_Position) : SV_Target
-        {
-            return t[int2(position.x, position.y)];
-        }
-#endif
-        0x43425844, 0x0beace24, 0x5e10b05b, 0x742de364, 0xb2b65d2b, 0x00000001, 0x00000140, 0x00000003,
-        0x0000002c, 0x00000060, 0x00000094, 0x4e475349, 0x0000002c, 0x00000001, 0x00000008, 0x00000020,
-        0x00000000, 0x00000001, 0x00000003, 0x00000000, 0x0000030f, 0x505f5653, 0x7469736f, 0x006e6f69,
-        0x4e47534f, 0x0000002c, 0x00000001, 0x00000008, 0x00000020, 0x00000000, 0x00000000, 0x00000003,
-        0x00000000, 0x00000e01, 0x545f5653, 0x65677261, 0xabab0074, 0x52444853, 0x000000a4, 0x00000040,
-        0x00000029, 0x04001858, 0x00107000, 0x00000000, 0x00005555, 0x04002064, 0x00101032, 0x00000000,
-        0x00000001, 0x03000065, 0x00102012, 0x00000000, 0x02000068, 0x00000001, 0x0500001b, 0x00100032,
-        0x00000000, 0x00101046, 0x00000000, 0x08000036, 0x001000c2, 0x00000000, 0x00004002, 0x00000000,
-        0x00000000, 0x00000000, 0x00000000, 0x0700002d, 0x001000f2, 0x00000000, 0x00100e46, 0x00000000,
-        0x00107e46, 0x00000000, 0x05000036, 0x00102012, 0x00000000, 0x0010000a, 0x00000000, 0x0100003e,
-    };
-    static const D3D12_SHADER_BYTECODE ps = {ps_code, sizeof(ps_code)};
-
-    static const DWORD ps_code_uint[] =
-    {
-#if 0
-        Texture2D<uint> t;
-
-        float main(float4 position : SV_Position) : SV_Target
-        {
-            return float(t[int2(position.x, position.y)]);
-        }
-#endif
-        0x43425844, 0x9a3fe38f, 0x3c222734, 0x1abb807b, 0xeb4ccda3, 0x00000001, 0x0000014c, 0x00000003,
-        0x0000002c, 0x00000060, 0x00000094, 0x4e475349, 0x0000002c, 0x00000001, 0x00000008, 0x00000020,
-        0x00000000, 0x00000001, 0x00000003, 0x00000000, 0x0000030f, 0x505f5653, 0x7469736f, 0x006e6f69,
-        0x4e47534f, 0x0000002c, 0x00000001, 0x00000008, 0x00000020, 0x00000000, 0x00000000, 0x00000003,
-        0x00000000, 0x00000e01, 0x545f5653, 0x65677261, 0xabab0074, 0x58454853, 0x000000b0, 0x00000050,
-        0x0000002c, 0x0100086a, 0x04001858, 0x00107000, 0x00000000, 0x00004444, 0x04002064, 0x00101032,
-        0x00000000, 0x00000001, 0x03000065, 0x00102012, 0x00000000, 0x02000068, 0x00000001, 0x0500001b,
-        0x00100032, 0x00000000, 0x00101046, 0x00000000, 0x08000036, 0x001000c2, 0x00000000, 0x00004002,
-        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8900002d, 0x800000c2, 0x00111103, 0x00100012,
-        0x00000000, 0x00100e46, 0x00000000, 0x00107e46, 0x00000000, 0x05000056, 0x00102012, 0x00000000,
-        0x0010000a, 0x00000000, 0x0100003e,
-    };
-    static const D3D12_SHADER_BYTECODE ps_uint = {ps_code_uint, sizeof(ps_code_uint)};
+#include "shaders/copy/headers/ps_copy_sample_float.h"
+#include "shaders/copy/headers/ps_copy_sample_uint.h"
 
     struct depth_copy_test
     {
@@ -262,9 +219,9 @@ void test_copy_texture(void)
     context.root_signature = create_texture_root_signature(device,
             D3D12_SHADER_VISIBILITY_PIXEL, 0, 0);
     pipeline_state_float = create_pipeline_state(device,
-            context.root_signature, context.render_target_desc.Format, NULL, &ps, NULL);
+            context.root_signature, context.render_target_desc.Format, NULL, &ps_copy_sample_float_dxbc, NULL);
     pipeline_state_uint = create_pipeline_state(device,
-            context.root_signature, context.render_target_desc.Format, NULL, &ps_uint, NULL);
+            context.root_signature, context.render_target_desc.Format, NULL, &ps_copy_sample_uint_dxbc, NULL);
 
     heap = create_gpu_descriptor_heap(device, D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV, 1);
 
@@ -1247,73 +1204,8 @@ void test_multisample_resolve_formats(void)
 
     static const float black[] = { 0.0f, 0.0f, 0.0f, 0.0f };
 
-    static const DWORD ps_setup_render_targets_code[] =
-    {
-#if 0
-    struct out_t
-    {
-        float d : SV_DEPTH;
-        float f : SV_TARGET0;
-        uint u : SV_TARGET1;
-        int s : SV_TARGET2;
-    };
-
-    out_t main(in float4 pos : SV_Position, in uint sample_id : SV_SampleIndex)
-    {
-        uint value = (sample_id ^ 1) + 4 * uint(pos.x) + 16 * uint(pos.y);
-        out_t result;
-        result.d = float(value) / 64.0f;
-        result.f = float(value) / 64.0f;
-        result.u = value;
-        result.s = int(value) - 32;
-        return result;
-    }
-#endif
-        0x43425844, 0xeb4fe5be, 0x516da9e0, 0x858b600f, 0x6a01a91d, 0x00000001, 0x0000028c, 0x00000003,
-        0x0000002c, 0x00000088, 0x0000010c, 0x4e475349, 0x00000054, 0x00000002, 0x00000008, 0x00000038,
-        0x00000000, 0x00000001, 0x00000003, 0x00000000, 0x0000030f, 0x00000044, 0x00000000, 0x0000000a,
-        0x00000001, 0x00000001, 0x00000101, 0x505f5653, 0x7469736f, 0x006e6f69, 0x535f5653, 0x6c706d61,
-        0x646e4965, 0xab007865, 0x4e47534f, 0x0000007c, 0x00000004, 0x00000008, 0x00000068, 0x00000000,
-        0x00000000, 0x00000003, 0x00000000, 0x00000e01, 0x00000068, 0x00000001, 0x00000000, 0x00000001,
-        0x00000001, 0x00000e01, 0x00000068, 0x00000002, 0x00000000, 0x00000002, 0x00000002, 0x00000e01,
-        0x00000072, 0x00000000, 0x00000000, 0x00000003, 0xffffffff, 0x00000e01, 0x545f5653, 0x45475241,
-        0x56530054, 0x5045445f, 0xab004854, 0x58454853, 0x00000178, 0x00000050, 0x0000005e, 0x0100086a,
-        0x04002064, 0x00101032, 0x00000000, 0x00000001, 0x04000863, 0x00101012, 0x00000001, 0x0000000a,
-        0x03000065, 0x00102012, 0x00000000, 0x03000065, 0x00102012, 0x00000001, 0x03000065, 0x00102012,
-        0x00000002, 0x02000065, 0x0000c001, 0x02000068, 0x00000001, 0x07000057, 0x00100012, 0x00000000,
-        0x0010100a, 0x00000001, 0x00004001, 0x00000001, 0x0500001c, 0x00100062, 0x00000000, 0x00101106,
-        0x00000000, 0x0a000029, 0x00100062, 0x00000000, 0x00100656, 0x00000000, 0x00004002, 0x00000000,
-        0x00000002, 0x00000004, 0x00000000, 0x0700001e, 0x00100012, 0x00000000, 0x0010001a, 0x00000000,
-        0x0010000a, 0x00000000, 0x0700001e, 0x00100012, 0x00000000, 0x0010002a, 0x00000000, 0x0010000a,
-        0x00000000, 0x05000056, 0x00100022, 0x00000000, 0x0010000a, 0x00000000, 0x07000038, 0x00100022,
-        0x00000000, 0x0010001a, 0x00000000, 0x00004001, 0x3c800000, 0x04000036, 0x0000c001, 0x0010001a,
-        0x00000000, 0x05000036, 0x00102012, 0x00000000, 0x0010001a, 0x00000000, 0x05000036, 0x00102012,
-        0x00000001, 0x0010000a, 0x00000000, 0x0700001e, 0x00102012, 0x00000002, 0x0010000a, 0x00000000,
-        0x00004001, 0xffffffe0, 0x0100003e,
-    };
-    static const D3D12_SHADER_BYTECODE ps_setup_render_targets = { ps_setup_render_targets_code, sizeof(ps_setup_render_targets_code) };
-
-    static const DWORD ps_setup_stencil_code[] =
-    {
-#if 0
-    cbuffer sample_mask_t : register(b0)
-    {
-        uint sample_mask;
-    };
-
-    uint main() : SV_Coverage
-    {
-        return sample_mask;
-    }
-#endif
-        0x43425844, 0x2fc829d6, 0x337f408a, 0x9ea69217, 0x4ae9bda5, 0x00000001, 0x000000b4, 0x00000003,
-        0x0000002c, 0x0000003c, 0x00000070, 0x4e475349, 0x00000008, 0x00000000, 0x00000008, 0x4e47534f,
-        0x0000002c, 0x00000001, 0x00000008, 0x00000020, 0x00000000, 0x00000000, 0x00000001, 0xffffffff,
-        0x00000e01, 0x435f5653, 0x7265766f, 0x00656761, 0x58454853, 0x0000003c, 0x00000050, 0x0000000f,
-        0x0100086a, 0x04000059, 0x00208e46, 0x00000000, 0x00000001, 0x02000065, 0x0000f000, 0x05000036,
-        0x0000f001, 0x0020800a, 0x00000000, 0x00000000, 0x0100003e,
-    };
-    static const D3D12_SHADER_BYTECODE ps_setup_stencil = { ps_setup_stencil_code, sizeof(ps_setup_stencil_code) };
+#include "shaders/copy/headers/ps_resolve_setup_rt.h"
+#include "shaders/copy/headers/ps_resolve_setup_stencil.h"
 
     static const D3D12_RESOLVE_MODE resolve_modes[] =
     {
@@ -1354,7 +1246,7 @@ void test_multisample_resolve_formats(void)
     ok(hr == S_OK, "Failed to create root signature, hr %#x.\n");
 
     init_pipeline_state_desc(&pipeline_desc, rs_setup_render_targets,
-            DXGI_FORMAT_UNKNOWN, NULL, &ps_setup_render_targets, NULL);
+            DXGI_FORMAT_UNKNOWN, NULL, &ps_resolve_setup_rt_dxbc, NULL);
     pipeline_desc.NumRenderTargets = 3;
     pipeline_desc.DSVFormat = DXGI_FORMAT_D32_FLOAT_S8X24_UINT;
     pipeline_desc.RTVFormats[0] = DXGI_FORMAT_R32_FLOAT;
@@ -1370,7 +1262,7 @@ void test_multisample_resolve_formats(void)
     ok(hr == S_OK, "Failed to create graphics pipeline, hr %#x.\n");
 
     init_pipeline_state_desc(&pipeline_desc, rs_setup_stencil,
-            DXGI_FORMAT_UNKNOWN, NULL, &ps_setup_stencil, NULL);
+            DXGI_FORMAT_UNKNOWN, NULL, &ps_resolve_setup_stencil_dxbc, NULL);
     pipeline_desc.DSVFormat = DXGI_FORMAT_D32_FLOAT_S8X24_UINT;
     pipeline_desc.SampleDesc.Count = 4;
     pipeline_desc.DepthStencilState.StencilEnable = TRUE;
@@ -1792,6 +1684,229 @@ void test_multisample_resolve_formats(void)
     destroy_test_context(&context);
 }
 
+void test_multisample_resolve_strongly_typed(void)
+{
+    D3D12_CPU_DESCRIPTOR_HANDLE src_rtv, dst_rtv;
+    D3D12_GRAPHICS_PIPELINE_STATE_DESC pso_desc;
+    ID3D12Resource *resolve_src, *resolve_dst;
+    D3D12_RENDER_TARGET_VIEW_DESC rtv_desc;
+    struct test_context_desc context_desc;
+    D3D12_HEAP_PROPERTIES heap_properties;
+    D3D12_ROOT_SIGNATURE_DESC rs_desc;
+    D3D12_RESOURCE_DESC resource_desc;
+    ID3D12DescriptorHeap *rtv_heap;
+    struct test_context context;
+    ID3D12PipelineState *pso;
+    D3D12_VIEWPORT viewport;
+    D3D12_RECT scissor;
+    unsigned int i;
+    HRESULT hr;
+
+#include "shaders/copy/headers/ps_resolve_setup_simple.h"
+
+    static const FLOAT red[] = { 1.0f, 0.0f, 0.0f, 1.0f };
+    static const FLOAT green[] = { 0.0f, 1.0f, 0.0f, 1.0f };
+
+    struct
+    {
+        DXGI_FORMAT dst_format;
+        DXGI_FORMAT src_format;
+        DXGI_FORMAT resolve_format;
+        uint32_t expected;
+    }
+    tests[] =
+    {
+        /* We can override the resolve format as long as it's compatible with the image */
+        { DXGI_FORMAT_R8G8B8A8_UNORM_SRGB, DXGI_FORMAT_R8G8B8A8_UNORM_SRGB, DXGI_FORMAT_R8G8B8A8_UNORM_SRGB, 0xffbcbcbc },
+        { DXGI_FORMAT_R8G8B8A8_UNORM_SRGB, DXGI_FORMAT_R8G8B8A8_UNORM_SRGB, DXGI_FORMAT_R8G8B8A8_UNORM, 0xff808080 },
+
+        /* Source and destination images may have different formats if compatible */
+        { DXGI_FORMAT_R8G8B8A8_UNORM_SRGB, DXGI_FORMAT_R8G8B8A8_UNORM, DXGI_FORMAT_R8G8B8A8_UNORM_SRGB, 0xffbcbcbc },
+        { DXGI_FORMAT_R8G8B8A8_UNORM_SRGB, DXGI_FORMAT_R8G8B8A8_UNORM, DXGI_FORMAT_R8G8B8A8_UNORM, 0xff808080 },
+
+        { DXGI_FORMAT_R8G8B8A8_UNORM, DXGI_FORMAT_R8G8B8A8_UNORM_SRGB, DXGI_FORMAT_R8G8B8A8_UNORM_SRGB, 0xffbcbcbc },
+        { DXGI_FORMAT_R8G8B8A8_UNORM, DXGI_FORMAT_R8G8B8A8_UNORM_SRGB, DXGI_FORMAT_R8G8B8A8_UNORM, 0xff808080 },
+    };
+
+    struct
+    {
+        DXGI_FORMAT dst_format;
+        DXGI_FORMAT src_format;
+        DXGI_FORMAT resolve_format;
+        HRESULT expected;
+        bool is_todo;
+    }
+    invalid_tests[] =
+    {
+        { DXGI_FORMAT_R8G8B8A8_UNORM, DXGI_FORMAT_R8G8B8A8_UNORM, DXGI_FORMAT_UNKNOWN, E_INVALIDARG },
+        { DXGI_FORMAT_R8G8B8A8_UNORM, DXGI_FORMAT_R8G8B8A8_UNORM, DXGI_FORMAT_R8G8B8A8_TYPELESS, E_INVALIDARG, true },
+        { DXGI_FORMAT_R8G8B8A8_UNORM, DXGI_FORMAT_R8G8B8A8_UNORM, DXGI_FORMAT_R32_FLOAT, E_INVALIDARG, true },
+
+        /* All of these are UB, but not validated by the runtime or debug layer */
+        { DXGI_FORMAT_R16_TYPELESS, DXGI_FORMAT_R16_FLOAT, DXGI_FORMAT_R16_UNORM, S_OK },
+        { DXGI_FORMAT_R16_TYPELESS, DXGI_FORMAT_R16_UNORM, DXGI_FORMAT_R16_FLOAT, S_OK },
+        { DXGI_FORMAT_R16_UNORM, DXGI_FORMAT_R16_TYPELESS, DXGI_FORMAT_R16_FLOAT, S_OK },
+        { DXGI_FORMAT_R16_FLOAT, DXGI_FORMAT_R16_TYPELESS, DXGI_FORMAT_R16_UNORM, S_OK },
+
+        { DXGI_FORMAT_R16_UNORM, DXGI_FORMAT_R16_FLOAT, DXGI_FORMAT_R16_FLOAT, S_OK },
+        { DXGI_FORMAT_R16_UNORM, DXGI_FORMAT_R16_FLOAT, DXGI_FORMAT_R16_UNORM, S_OK },
+    };
+
+    memset(&context_desc, 0, sizeof(context_desc));
+    context_desc.no_render_target = true;
+    context_desc.no_pipeline = true;
+    if (!init_test_context(&context, &context_desc))
+        return;
+
+    memset(&rs_desc, 0, sizeof(rs_desc));
+    hr = create_root_signature(context.device, &rs_desc, &context.root_signature);
+    ok(hr == S_OK, "Failed to create root signature, hr %#x.\n");
+
+    init_pipeline_state_desc(&pso_desc, context.root_signature,
+            DXGI_FORMAT_UNKNOWN, NULL, &ps_resolve_setup_simple_dxbc, NULL);
+    pso_desc.NumRenderTargets = 1;
+    pso_desc.SampleDesc.Count = 4;
+
+    rtv_heap = create_cpu_descriptor_heap(context.device, D3D12_DESCRIPTOR_HEAP_TYPE_RTV, 2);
+
+    dst_rtv = get_cpu_rtv_handle(&context, rtv_heap, 0);
+    src_rtv = get_cpu_rtv_handle(&context, rtv_heap, 1);
+
+    memset(&heap_properties, 0, sizeof(heap_properties));
+    heap_properties.Type = D3D12_HEAP_TYPE_DEFAULT;
+
+    viewport.TopLeftX = 0.0f;
+    viewport.TopLeftY = 0.0f;
+    viewport.Width = 4.0f;
+    viewport.Height = 4.0f;
+    viewport.MinDepth = 0.0f;
+    viewport.MaxDepth = 0.0f;
+
+    scissor.left = 0;
+    scissor.top = 0;
+    scissor.right = 4;
+    scissor.bottom = 4;
+
+    for (i = 0; i < ARRAY_SIZE(tests); i++)
+    {
+        vkd3d_test_set_context("Test %u", i);
+
+        memset(&resource_desc, 0, sizeof(resource_desc));
+        resource_desc.Dimension = D3D12_RESOURCE_DIMENSION_TEXTURE2D;
+        resource_desc.Width = 4;
+        resource_desc.Height = 4;
+        resource_desc.DepthOrArraySize = 1;
+        resource_desc.MipLevels = 1;
+        resource_desc.SampleDesc.Count = 4;
+        resource_desc.Format = tests[i].src_format;
+        resource_desc.Layout = D3D12_TEXTURE_LAYOUT_UNKNOWN;
+        resource_desc.Flags = D3D12_RESOURCE_FLAG_ALLOW_RENDER_TARGET;
+
+        hr = ID3D12Device_CreateCommittedResource(context.device, &heap_properties, D3D12_HEAP_FLAG_NONE,
+                &resource_desc, D3D12_RESOURCE_STATE_RENDER_TARGET, NULL, &IID_ID3D12Resource, (void**)&resolve_src);
+        ok(hr == S_OK, "Failed to create resolve source.\n");
+
+        resource_desc.SampleDesc.Count = 1;
+        resource_desc.Format = tests[i].dst_format;
+
+        hr = ID3D12Device_CreateCommittedResource(context.device, &heap_properties, D3D12_HEAP_FLAG_NONE,
+                &resource_desc, D3D12_RESOURCE_STATE_RENDER_TARGET, NULL, &IID_ID3D12Resource, (void**)&resolve_dst);
+        ok(hr == S_OK, "Failed to create resolve destination.\n");
+
+        memset(&rtv_desc, 0, sizeof(rtv_desc));
+        rtv_desc.ViewDimension = D3D12_RTV_DIMENSION_TEXTURE2DMS;
+        rtv_desc.Format = tests[i].src_format;
+
+        ID3D12Device_CreateRenderTargetView(context.device, resolve_src, &rtv_desc, src_rtv);
+
+        rtv_desc.ViewDimension = D3D12_RTV_DIMENSION_TEXTURE2D;
+        rtv_desc.Format = tests[i].dst_format;
+
+        ID3D12Device_CreateRenderTargetView(context.device, resolve_dst, &rtv_desc, dst_rtv);
+
+        pso_desc.RTVFormats[0] = tests[i].src_format;
+
+        hr = ID3D12Device_CreateGraphicsPipelineState(context.device, &pso_desc, &IID_ID3D12PipelineState, (void**)&pso);
+        ok(hr == S_OK, "Failed to create graphics pipeline, hr %#x.\n");
+
+        ID3D12GraphicsCommandList_ClearRenderTargetView(context.list, src_rtv, green, 0, NULL);
+        ID3D12GraphicsCommandList_ClearRenderTargetView(context.list, dst_rtv, red, 0, NULL);
+
+        ID3D12GraphicsCommandList_OMSetRenderTargets(context.list, 1, &src_rtv, TRUE, NULL);
+        ID3D12GraphicsCommandList_SetGraphicsRootSignature(context.list, context.root_signature);
+        ID3D12GraphicsCommandList_SetPipelineState(context.list, pso);
+        ID3D12GraphicsCommandList_IASetPrimitiveTopology(context.list, D3D_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
+        ID3D12GraphicsCommandList_RSSetViewports(context.list, 1, &viewport);
+        ID3D12GraphicsCommandList_RSSetScissorRects(context.list, 1, &scissor);
+        ID3D12GraphicsCommandList_DrawInstanced(context.list, 3, 1, 0, 0);
+
+        transition_resource_state(context.list, resolve_dst, D3D12_RESOURCE_STATE_RENDER_TARGET, D3D12_RESOURCE_STATE_RESOLVE_DEST);
+        transition_resource_state(context.list, resolve_src, D3D12_RESOURCE_STATE_RENDER_TARGET, D3D12_RESOURCE_STATE_RESOLVE_SOURCE);
+
+        ID3D12GraphicsCommandList_ResolveSubresource(context.list, resolve_dst, 0,
+                resolve_src, 0, tests[i].resolve_format);
+
+        transition_resource_state(context.list, resolve_dst, D3D12_RESOURCE_STATE_RESOLVE_DEST, D3D12_RESOURCE_STATE_COPY_SOURCE);
+
+        check_sub_resource_uint(resolve_dst, 0, context.queue, context.list, tests[i].expected, 1);
+        reset_command_list(context.list, context.allocator);
+
+        ID3D12PipelineState_Release(pso);
+
+        ID3D12Resource_Release(resolve_dst);
+        ID3D12Resource_Release(resolve_src);
+    }
+
+    for (i = 0; i < ARRAY_SIZE(invalid_tests); i++)
+    {
+        vkd3d_test_set_context("Test %u", i);
+
+        memset(&resource_desc, 0, sizeof(resource_desc));
+        resource_desc.Dimension = D3D12_RESOURCE_DIMENSION_TEXTURE2D;
+        resource_desc.Width = 4;
+        resource_desc.Height = 4;
+        resource_desc.DepthOrArraySize = 1;
+        resource_desc.MipLevels = 1;
+        resource_desc.SampleDesc.Count = 4;
+        resource_desc.Format = invalid_tests[i].src_format;
+        resource_desc.Layout = D3D12_TEXTURE_LAYOUT_UNKNOWN;
+        resource_desc.Flags = D3D12_RESOURCE_FLAG_ALLOW_RENDER_TARGET;
+
+        hr = ID3D12Device_CreateCommittedResource(context.device, &heap_properties, D3D12_HEAP_FLAG_NONE,
+                &resource_desc, D3D12_RESOURCE_STATE_RESOLVE_SOURCE, NULL, &IID_ID3D12Resource, (void**)&resolve_src);
+        ok(hr == S_OK, "Failed to create resolve source.\n");
+
+        resource_desc.SampleDesc.Count = 1;
+        resource_desc.Format = invalid_tests[i].dst_format;
+
+        hr = ID3D12Device_CreateCommittedResource(context.device, &heap_properties, D3D12_HEAP_FLAG_NONE,
+                &resource_desc, D3D12_RESOURCE_STATE_RESOLVE_DEST, NULL, &IID_ID3D12Resource, (void**)&resolve_dst);
+        ok(hr == S_OK, "Failed to create resolve destination, hr %#x.\n", hr);
+
+        ID3D12GraphicsCommandList_ResolveSubresource(context.list, resolve_dst, 0,
+                resolve_src, 0, invalid_tests[i].resolve_format);
+
+        hr = ID3D12GraphicsCommandList_Close(context.list);
+        todo_if(invalid_tests[i].is_todo)
+        ok(hr == invalid_tests[i].expected, "Got hr %#x, expected E_INVALIDARG.\n", hr);
+        ID3D12GraphicsCommandList_Release(context.list);
+
+        hr = ID3D12CommandAllocator_Reset(context.allocator);
+        ok(hr == S_OK, "Failed to reset command allocator, hr %#x.\n", hr);
+
+        hr = ID3D12Device_CreateCommandList(context.device, 0, D3D12_COMMAND_LIST_TYPE_DIRECT,
+                context.allocator, NULL, &IID_ID3D12GraphicsCommandList, (void**)&context.list);
+        ok(hr == S_OK, "Failed to create command list, hr %#x.\n", hr);
+
+        ID3D12Resource_Release(resolve_dst);
+        ID3D12Resource_Release(resolve_src);
+    }
+
+    ID3D12DescriptorHeap_Release(rtv_heap);
+
+    destroy_test_context(&context);
+}
+
 void test_copy_buffer_overlap(void)
 {
     uint32_t reference_output[4][16 * 1024] = {{0}};
