@@ -107,6 +107,7 @@ void test_check_feature_support(void)
 
     static const D3D_FEATURE_LEVEL all_feature_levels[] =
     {
+        D3D_FEATURE_LEVEL_12_2,
         D3D_FEATURE_LEVEL_12_1,
         D3D_FEATURE_LEVEL_12_0,
         D3D_FEATURE_LEVEL_11_1,
@@ -119,6 +120,7 @@ void test_check_feature_support(void)
     };
     static const D3D_FEATURE_LEVEL d3d12_feature_levels[] =
     {
+        D3D_FEATURE_LEVEL_12_2,
         D3D_FEATURE_LEVEL_12_1,
         D3D_FEATURE_LEVEL_12_0,
         D3D_FEATURE_LEVEL_11_1,
@@ -299,6 +301,20 @@ void test_check_feature_support(void)
             || root_signature.HighestVersion == D3D_ROOT_SIGNATURE_VERSION_1_1,
             "Got unexpected root signature feature version %#x.\n", root_signature.HighestVersion);
 
+    root_signature.HighestVersion = 0;
+    hr = ID3D12Device_CheckFeatureSupport(device, D3D12_FEATURE_ROOT_SIGNATURE,
+            &root_signature, sizeof(root_signature));
+    ok(hr == E_INVALIDARG, "Got unexpected hr %#x.\n", hr);
+    ok(root_signature.HighestVersion == 0, "Got unexpected root signature feature version %#x.\n",
+            root_signature.HighestVersion);
+
+    root_signature.HighestVersion = 0xdeadbeef;
+    hr = ID3D12Device_CheckFeatureSupport(device, D3D12_FEATURE_ROOT_SIGNATURE,
+            &root_signature, sizeof(root_signature));
+    ok(hr == E_INVALIDARG, "Got unexpected hr %#x.\n", hr);
+    ok(root_signature.HighestVersion == 0xdeadbeef, "Got unexpected root signature feature version %#x.\n",
+            root_signature.HighestVersion);
+
     refcount = ID3D12Device_Release(device);
     ok(!refcount, "ID3D12Device has %u references left.\n", (unsigned int)refcount);
 }
@@ -322,11 +338,96 @@ static const DXGI_FORMAT depth_stencil_formats[] =
 void test_format_support(void)
 {
     D3D12_FEATURE_DATA_FORMAT_SUPPORT format_support;
+    D3D12_FEATURE_DATA_FEATURE_LEVELS feature_levels;
+    bool required_but_fails;
+    bool unspecified_format;
+    bool optional_format;
     ID3D12Device *device;
+    DXGI_FORMAT format;
     ULONG refcount;
     unsigned int i;
     HRESULT hr;
 
+    static const DXGI_FORMAT known_required_but_fails[] =
+    {
+        DXGI_FORMAT_R8G8_B8G8_UNORM,
+        DXGI_FORMAT_G8R8_G8B8_UNORM,
+        DXGI_FORMAT_R10G10B10_XR_BIAS_A2_UNORM,
+        DXGI_FORMAT_NV12,
+        DXGI_FORMAT_420_OPAQUE,
+        DXGI_FORMAT_YUY2,
+    };
+
+#define MAX_FORMAT_VALUE DXGI_FORMAT_SAMPLER_FEEDBACK_MIP_REGION_USED_OPAQUE
+
+    const struct dxgi_format_list
+    {
+        D3D_FEATURE_LEVEL feature_level;
+        DXGI_FORMAT optional_formats[MAX_FORMAT_VALUE];
+        DXGI_FORMAT unspecified_formats[MAX_FORMAT_VALUE];
+    } *dxgi_format, dxgi_format_list[] = {
+        /*
+            https://devblogs.microsoft.com/directx/new-in-directx-feature-level-12_2/
+            https://microsoft.github.io/DirectX-Specs/d3d/SamplerFeedback.html
+        */
+        {D3D_FEATURE_LEVEL_12_2, 
+            {DXGI_FORMAT_AYUV, DXGI_FORMAT_Y410, DXGI_FORMAT_Y416, DXGI_FORMAT_P010, DXGI_FORMAT_P016, DXGI_FORMAT_Y210, DXGI_FORMAT_Y216, DXGI_FORMAT_NV11,
+                DXGI_FORMAT_AI44, DXGI_FORMAT_IA44, DXGI_FORMAT_P8, DXGI_FORMAT_A8P8},
+            {DXGI_FORMAT_R1_UNORM, DXGI_FORMAT_B4G4R4A4_UNORM, DXGI_FORMAT_P208, DXGI_FORMAT_V208, DXGI_FORMAT_V408}},
+        /* https://github.com/MicrosoftDocs/win32/blob/docs/desktop-src/direct3ddxgi/hardware-support-for-direct3d-12-1-formats.md */
+        {D3D_FEATURE_LEVEL_12_1, 
+            {DXGI_FORMAT_AYUV, DXGI_FORMAT_Y410, DXGI_FORMAT_Y416, DXGI_FORMAT_P010, DXGI_FORMAT_P016, DXGI_FORMAT_Y210, DXGI_FORMAT_Y216, DXGI_FORMAT_NV11,
+                DXGI_FORMAT_AI44, DXGI_FORMAT_IA44, DXGI_FORMAT_P8, DXGI_FORMAT_A8P8},
+            {DXGI_FORMAT_R1_UNORM, DXGI_FORMAT_B4G4R4A4_UNORM, DXGI_FORMAT_P208, DXGI_FORMAT_V208, DXGI_FORMAT_V408, 
+                DXGI_FORMAT_SAMPLER_FEEDBACK_MIN_MIP_OPAQUE, DXGI_FORMAT_SAMPLER_FEEDBACK_MIP_REGION_USED_OPAQUE}},
+        /* https://github.com/MicrosoftDocs/win32/blob/docs/desktop-src/direct3ddxgi/hardware-support-for-direct3d-12-0-formats.md */
+        {D3D_FEATURE_LEVEL_12_0, 
+            {DXGI_FORMAT_AYUV, DXGI_FORMAT_Y410, DXGI_FORMAT_Y416, DXGI_FORMAT_P010, DXGI_FORMAT_P016, DXGI_FORMAT_Y210, DXGI_FORMAT_Y216, DXGI_FORMAT_NV11,
+                DXGI_FORMAT_AI44, DXGI_FORMAT_IA44, DXGI_FORMAT_P8, DXGI_FORMAT_A8P8},
+            {DXGI_FORMAT_R1_UNORM, DXGI_FORMAT_P208, DXGI_FORMAT_V208, DXGI_FORMAT_V408, 
+                DXGI_FORMAT_SAMPLER_FEEDBACK_MIN_MIP_OPAQUE, DXGI_FORMAT_SAMPLER_FEEDBACK_MIP_REGION_USED_OPAQUE}},
+        /* https://github.com/MicrosoftDocs/win32/blob/docs/desktop-src/direct3ddxgi/format-support-for-direct3d-11-1-feature-level-hardware.md */
+        {D3D_FEATURE_LEVEL_11_1, 
+            {DXGI_FORMAT_AYUV, DXGI_FORMAT_Y410, DXGI_FORMAT_Y416, DXGI_FORMAT_P010, DXGI_FORMAT_P016, DXGI_FORMAT_Y210, DXGI_FORMAT_Y216, DXGI_FORMAT_NV11,
+                DXGI_FORMAT_AI44, DXGI_FORMAT_IA44, DXGI_FORMAT_P8, DXGI_FORMAT_A8P8}, 
+            {DXGI_FORMAT_R1_UNORM, DXGI_FORMAT_P208, DXGI_FORMAT_V208, DXGI_FORMAT_V408, 
+                DXGI_FORMAT_SAMPLER_FEEDBACK_MIN_MIP_OPAQUE, DXGI_FORMAT_SAMPLER_FEEDBACK_MIP_REGION_USED_OPAQUE}},
+        /* https://github.com/MicrosoftDocs/win32/blob/docs/desktop-src/direct3ddxgi/format-support-for-direct3d-11-0-feature-level-hardware.md */
+        {D3D_FEATURE_LEVEL_11_0, 
+            {DXGI_FORMAT_AYUV, DXGI_FORMAT_Y410, DXGI_FORMAT_Y416, DXGI_FORMAT_P010, DXGI_FORMAT_P016, DXGI_FORMAT_Y210, DXGI_FORMAT_Y216, DXGI_FORMAT_NV11,
+                DXGI_FORMAT_AI44, DXGI_FORMAT_IA44, DXGI_FORMAT_P8, DXGI_FORMAT_A8P8}, 
+            {DXGI_FORMAT_R1_UNORM, DXGI_FORMAT_P208, DXGI_FORMAT_V208, DXGI_FORMAT_V408, 
+                DXGI_FORMAT_SAMPLER_FEEDBACK_MIN_MIP_OPAQUE, DXGI_FORMAT_SAMPLER_FEEDBACK_MIP_REGION_USED_OPAQUE}},
+        /* https://github.com/MicrosoftDocs/win32/blob/docs/desktop-src/direct3ddxgi/format-support-for-direct3d-feature-level-10-1-hardware.md */
+        {D3D_FEATURE_LEVEL_10_1, 
+            {DXGI_FORMAT_AYUV, DXGI_FORMAT_Y410, DXGI_FORMAT_Y416, DXGI_FORMAT_P010, DXGI_FORMAT_P016, DXGI_FORMAT_Y210, 
+                DXGI_FORMAT_Y216, DXGI_FORMAT_NV11, DXGI_FORMAT_AI44, DXGI_FORMAT_IA44, DXGI_FORMAT_P8, DXGI_FORMAT_A8P8}, 
+            {DXGI_FORMAT_R1_UNORM, DXGI_FORMAT_P208, DXGI_FORMAT_V208, DXGI_FORMAT_V408, 
+                DXGI_FORMAT_SAMPLER_FEEDBACK_MIN_MIP_OPAQUE, DXGI_FORMAT_SAMPLER_FEEDBACK_MIP_REGION_USED_OPAQUE}},
+        /* https://github.com/MicrosoftDocs/win32/blob/docs/desktop-src/direct3ddxgi/format-support-for-direct3d-feature-level-10-1-hardware.md */
+        {D3D_FEATURE_LEVEL_10_0, 
+            {DXGI_FORMAT_R10G10B10_XR_BIAS_A2_UNORM, DXGI_FORMAT_AYUV, DXGI_FORMAT_Y410, DXGI_FORMAT_Y416, DXGI_FORMAT_P010, DXGI_FORMAT_P016, 
+                DXGI_FORMAT_Y210, DXGI_FORMAT_Y216, DXGI_FORMAT_NV11, DXGI_FORMAT_AI44, DXGI_FORMAT_IA44, DXGI_FORMAT_P8, DXGI_FORMAT_A8P8}, 
+            {DXGI_FORMAT_R1_UNORM, DXGI_FORMAT_P208, DXGI_FORMAT_V208, DXGI_FORMAT_V408, 
+                DXGI_FORMAT_SAMPLER_FEEDBACK_MIN_MIP_OPAQUE, DXGI_FORMAT_SAMPLER_FEEDBACK_MIP_REGION_USED_OPAQUE}},
+    };
+
+#undef MAX_FORMAT_VALUE
+
+    static const D3D_FEATURE_LEVEL all_feature_levels[] =
+    {
+        D3D_FEATURE_LEVEL_12_2,
+        D3D_FEATURE_LEVEL_12_1,
+        D3D_FEATURE_LEVEL_12_0,
+        D3D_FEATURE_LEVEL_11_1,
+        D3D_FEATURE_LEVEL_11_0,
+        D3D_FEATURE_LEVEL_10_1,
+        D3D_FEATURE_LEVEL_10_0,
+        D3D_FEATURE_LEVEL_9_3,
+        D3D_FEATURE_LEVEL_9_2,
+        D3D_FEATURE_LEVEL_9_1,
+    };
+
     if (!(device = create_device()))
     {
         skip("Failed to create device.\n");
@@ -336,20 +437,104 @@ void test_format_support(void)
     memset(&format_support, 0, sizeof(format_support));
     hr = ID3D12Device_CheckFeatureSupport(device, D3D12_FEATURE_FORMAT_SUPPORT,
             &format_support, sizeof(format_support));
-    todo ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
-    todo ok(format_support.Support1 == D3D12_FORMAT_SUPPORT1_BUFFER,
+    ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
+    ok(format_support.Support1 == D3D12_FORMAT_SUPPORT1_BUFFER,
             "Got unexpected support1 %#x.\n", format_support.Support1);
     ok(!format_support.Support2 || format_support.Support2 == D3D12_FORMAT_SUPPORT2_TILED,
             "Got unexpected support2 %#x.\n", format_support.Support2);
 
-    for (i = 0; i < ARRAY_SIZE(depth_stencil_formats); ++i)
+    memset(&feature_levels, 0, sizeof(feature_levels));
+    feature_levels.NumFeatureLevels = ARRAY_SIZE(all_feature_levels);
+    feature_levels.pFeatureLevelsRequested = all_feature_levels;
+    hr = ID3D12Device_CheckFeatureSupport(device, D3D12_FEATURE_FEATURE_LEVELS, 
+        &feature_levels, sizeof(feature_levels));
+    ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
+
+    dxgi_format = NULL;
+    for (i = 0; i < ARRAY_SIZE(dxgi_format_list); ++i)
     {
+        if (dxgi_format_list[i].feature_level == feature_levels.MaxSupportedFeatureLevel)
+        {
+            dxgi_format = &dxgi_format_list[i];
+            break;
+        }
+    }
+    assert(dxgi_format);
+
+    for (format = 0; format <= DXGI_FORMAT_SAMPLER_FEEDBACK_MIP_REGION_USED_OPAQUE; ++format)
+    {
+        /* Undefined range, skip */
+        if (format > DXGI_FORMAT_B4G4R4A4_UNORM && format < DXGI_FORMAT_P208)
+            continue;
+
+        vkd3d_test_set_context("format %#x", format);
+
+        required_but_fails = false;
+        unspecified_format = false;
+        optional_format = false;
+
+        for (i = 0; i < ARRAY_SIZE(known_required_but_fails); ++i)
+        {
+            if (known_required_but_fails[i] == format)
+            {
+                required_but_fails = true;
+                break;
+            }
+        }
+
+        if (!required_but_fails)
+        {
+            /* ASTC format are unspecified */
+            if (format >= /* DXGI_FORMAT_UNDOCUMENTED_ASTC_FIRST */ (DXGI_FORMAT)0x85 && 
+                format <= /* DXGI_FORMAT_UNDOCUMENTED_ASTC_LAST */  (DXGI_FORMAT)0xbc)
+            {
+                unspecified_format = true;
+            }
+            else
+            {
+                /* Check if the format is unspecified or optional */
+                for (i = 0; i < ARRAY_SIZE(dxgi_format->unspecified_formats); ++i)
+                {
+                    /* Fixed size list with only part of the list filled */
+                    if (dxgi_format->unspecified_formats[i] == DXGI_FORMAT_UNKNOWN)
+                        break;
+                    if (dxgi_format->unspecified_formats[i] == format)
+                    {
+                        unspecified_format = true;
+                        break;
+                    }
+                }
+
+                if (!unspecified_format)
+                {
+                    for (i = 0; i < ARRAY_SIZE(dxgi_format->optional_formats); ++i)
+                    {
+                        /* Fixed size list with only part of the list filled */
+                        if (dxgi_format->optional_formats[i] == DXGI_FORMAT_UNKNOWN)
+                            break;
+                        if (dxgi_format->optional_formats[i] == format)
+                        {
+                            optional_format = true;
+                            break;
+                        }
+                    }
+                }
+            }
+        }
+
         memset(&format_support, 0, sizeof(format_support));
-        format_support.Format = depth_stencil_formats[i];
+        format_support.Format = format;
         hr = ID3D12Device_CheckFeatureSupport(device, D3D12_FEATURE_FORMAT_SUPPORT,
                 &format_support, sizeof(format_support));
-        ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
+
+        if (unspecified_format)
+            ok(hr == S_OK || hr == E_FAIL, "Unspecified format %d got unexpected hr %#x.\n", format, hr);
+        else if (optional_format)
+            ok(hr == S_OK || hr == E_FAIL, "Optional format %d got unexpected hr %#x.\n", format, hr);
+        else 
+            todo_if(required_but_fails) ok(hr == S_OK, "Format %d got unexpected hr %#x.\n", format, hr);
     }
+    vkd3d_test_set_context(NULL);
 
     refcount = ID3D12Device_Release(device);
     ok(!refcount, "ID3D12Device has %u references left.\n", (unsigned int)refcount);
@@ -1074,16 +1259,54 @@ void test_reset_command_allocator(void)
                 command_allocator, NULL, &IID_ID3D12GraphicsCommandList, (void **)&command_list2);
         ok(hr == S_OK, "Failed to create command list, hr %#x.\n", hr);
 
-        ID3D12CommandAllocator_Release(command_allocator);
-        ID3D12CommandAllocator_Release(command_allocator2);
         ID3D12GraphicsCommandList_Release(command_list);
         ID3D12GraphicsCommandList_Release(command_list2);
+        ID3D12CommandAllocator_Release(command_allocator);
+        ID3D12CommandAllocator_Release(command_allocator2);
     }
 
     refcount = ID3D12Device_Release(device);
     ok(!refcount, "ID3D12Device has %u references left.\n", (unsigned int)refcount);
 }
 
+void test_object_interface_null_cases(void)
+{
+    ID3D12Device *device;
+    ID3D12Fence *fence;
+    HRESULT hr;
+    UINT ref;
+
+    if (!(device = create_device()))
+    {
+        skip("Failed to create device.\n");
+        return;
+    }
+
+    hr = ID3D12Device_CreateFence(device, 0, D3D12_FENCE_FLAG_NONE, &IID_ID3D12Fence, (void **)&fence);
+    ok(SUCCEEDED(hr), "Failed to create fence, hr #%x.\n", hr);
+
+    hr = ID3D12Device_CreateFence(device, 0, D3D12_FENCE_FLAG_NONE, &IID_ID3D12Fence, NULL);
+    ok(hr == S_FALSE, "Unexpected hr #%x.\n", hr);
+
+    /* S_FALSE is returned even for bogus requested interfaces. */
+    hr = ID3D12Device_CreateFence(device, 0, D3D12_FENCE_FLAG_NONE, &IID_ID3D12Heap, NULL);
+    ok(hr == S_FALSE, "Unexpected hr #%x.\n", hr);
+
+    /* E_POINTER is always returned if QueryInterface ppOutput is NULL. */
+    hr = ID3D12Fence_QueryInterface(fence, &IID_IUnknown, NULL);
+    ok(hr == E_POINTER, "Unexpected hr #%x.\n", hr);
+
+    hr = ID3D12Fence_QueryInterface(fence, &IID_ID3D12Fence, NULL);
+    ok(hr == E_POINTER, "Unexpected hr #%x.\n", hr);
+
+    hr = ID3D12Fence_QueryInterface(fence, &IID_ID3D12Heap, NULL);
+    ok(hr == E_POINTER, "Unexpected hr #%x.\n", hr);
+
+    ID3D12Fence_Release(fence);
+    ref = ID3D12Device_Release(device);
+    ok(ref == 0, "Unexpected ref-count %u\n", ref);
+}
+
 void test_object_interface(void)
 {
     D3D12_DESCRIPTOR_HEAP_DESC descriptor_heap_desc;
@@ -1354,11 +1577,14 @@ void test_object_interface(void)
         hr = ID3D12Object_GetPrivateData(object, &WKPDID_D3DDebugObjectName, &size, NULL);
         ok(hr == DXGI_ERROR_NOT_FOUND, "Got unexpected hr %#x.\n", hr);
 
+#if 0
+        /* NULL name crashes on Windows 11 22621. */
         hr = ID3D12Object_SetName(object, NULL);
         ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
 
         hr = ID3D12Object_GetPrivateData(object, &WKPDID_D3DDebugObjectNameW, &size, NULL);
         ok(hr == DXGI_ERROR_NOT_FOUND, "Got unexpected hr %#x.\n", hr);
+#endif
 
         hr = ID3D12Object_SetPrivateData(object, &WKPDID_D3DDebugObjectName, sizeof(terminated_name_a), terminated_name_a);
         ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
@@ -1444,3 +1670,103 @@ void test_device_removed_reason(void)
     ok(!refcount, "ID3D12Device has %u references left.\n", (unsigned int)refcount);
 }
 
+void test_enumerate_meta_commands(void)
+{
+    D3D12_META_COMMAND_DESC dummy_desc, *descs;
+    UINT desc_count, supported_count;
+    ID3D12Device5 *device5;
+    ID3D12Device *device;
+    ULONG refcount;
+    HRESULT hr;
+
+    if (!(device = create_device()))
+    {
+        skip("Failed to create device.\n");
+        return;
+    }
+
+    hr = ID3D12Device_QueryInterface(device, &IID_ID3D12Device5, (void**)&device5);
+    ID3D12Device_Release(device);
+
+    if (FAILED(hr))
+    {
+        skip("ID3D12Device5 not supported by implementation.\n");
+        return;
+    }
+
+    hr = ID3D12Device5_EnumerateMetaCommands(device5, NULL, NULL);
+    ok(hr == E_INVALIDARG, "Unexpected hr %#x.\n", hr);
+    hr = ID3D12Device5_EnumerateMetaCommands(device5, NULL, &dummy_desc);
+    ok(hr == E_INVALIDARG, "Unexpected hr %#x.\n", hr);
+
+    desc_count = 0xdeadbeef;
+    hr = ID3D12Device5_EnumerateMetaCommands(device5, &desc_count, NULL);
+    ok(hr == S_OK, "Unexpected hr %#x.\n", hr);
+
+    supported_count = desc_count;
+
+    descs = calloc(supported_count, sizeof(*descs));
+    memset(&dummy_desc, 0, sizeof(dummy_desc));
+
+    hr = ID3D12Device5_EnumerateMetaCommands(device5, &desc_count, descs);
+    ok(hr == S_OK, "Unexpected hr %#x.\n", hr);
+    ok(desc_count == supported_count, "Unexpected desc count %u.\n", desc_count);
+
+    if (desc_count)
+    {
+        desc_count -= 1;
+
+        hr = ID3D12Device5_EnumerateMetaCommands(device5, &desc_count, descs);
+        ok(hr == S_OK, "Unexpected hr %#x.\n", hr);
+        ok(desc_count == supported_count, "Unexpected desc count %u.\n", desc_count);
+    }
+
+    /* The D3D12 runtime will crash or access random data if given a larger
+     * count than what is supported by the device, so don't test that here. */
+
+    free(descs);
+
+    refcount = ID3D12Device5_Release(device5);
+    ok(!refcount, "ID3D12Device has %u references left.\n", (unsigned int)refcount);
+}
+
+void test_vtable_origins(void)
+{
+#ifdef _WIN32
+    HMODULE d3d12core_module, vtable_module;
+    MEMORY_BASIC_INFORMATION info;
+    ID3D12Device *device;
+    SIZE_T ret;
+
+    if (!(device = create_device()))
+    {
+        skip("Failed to create device.\n");
+        return;
+    }
+
+    /* Skip this test if running on an older version of
+     * Windows from before the D3D12 + D3D12Core split. */
+    if (!(d3d12core_module = GetModuleHandleA("d3d12core")))
+    {
+        skip("No D3D12Core module, skipping.\n");
+        ID3D12Device_Release(device);
+        return;
+    }
+
+    ret = VirtualQuery(device->lpVtbl, &info, sizeof(info));
+    ok(ret, "VirtualQuery of ID3D12Device VTable failed.\n");
+    if (!ret)
+    {
+        skip("VirtualQuery failed, skipping vtable test.\n");
+        ID3D12Device_Release(device);
+        return;
+    }
+
+    vtable_module = (HMODULE)info.AllocationBase;
+
+    /* Ensure the vtable for ID3D12Device comes from D3D12Core.dll */
+    ok(vtable_module == d3d12core_module, "VTable for ID3D12Device not provided by D3D12Core.\n");
+
+    ID3D12Device_Release(device);
+#endif
+}
