@@ -1187,3 +1187,503 @@ void test_create_fence(void)
     ok(!refcount, "ID3D12Device has %u references left.\n", (unsigned int)refcount);
 }
 
+void test_fence_wait_robustness_inner(bool shared_handles)
+{
+    VKD3D_UNUSED HANDLE shared_signal = NULL;
+    VKD3D_UNUSED HANDLE shared_drain = NULL;
+    VKD3D_UNUSED HANDLE shared_wait = NULL;
+    ID3D12CommandAllocator *allocator[2];
+    ID3D12Fence *signal_fence_dup = NULL;
+    D3D12_COMMAND_QUEUE_DESC queue_desc;
+    ID3D12Fence *drain_fence_dup = NULL;
+    ID3D12Fence *wait_fence_dup = NULL;
+    ID3D12GraphicsCommandList *list[2];
+    ID3D12CommandQueue *compute_queue;
+    struct test_context context;
+    ID3D12Fence *signal_fence;
+    ID3D12Fence *drain_fence;
+    ID3D12Fence *wait_fence;
+    ID3D12Resource *src;
+    ID3D12Resource *dst;
+    unsigned int i;
+    HANDLE event;
+    UINT value;
+    HRESULT hr;
+
+    if (!init_compute_test_context(&context))
+        return;
+
+    hr = ID3D12Device_CreateFence(context.device, 0,
+            shared_handles ? D3D12_FENCE_FLAG_SHARED : D3D12_FENCE_FLAG_NONE,
+            &IID_ID3D12Fence, (void**)&signal_fence);
+    todo_if(shared_handles) ok(SUCCEEDED(hr), "Failed to create fence, hr #%x.\n", hr);
+
+    if (FAILED(hr))
+    {
+        skip("Failed to create fence, skipping test ...\n");
+        destroy_test_context(&context);
+        return;
+    }
+
+    hr = ID3D12Device_CreateFence(context.device, 0,
+            shared_handles ? D3D12_FENCE_FLAG_SHARED : D3D12_FENCE_FLAG_NONE,
+            &IID_ID3D12Fence, (void**)&wait_fence);
+    ok(SUCCEEDED(hr), "Failed to create fence, hr #%x.\n", hr);
+
+    if (FAILED(hr))
+    {
+        skip("Failed to create fence, skipping test ...\n");
+        ID3D12Fence_Release(signal_fence);
+        destroy_test_context(&context);
+        return;
+    }
+
+    hr = ID3D12Device_CreateFence(context.device, 0,
+            shared_handles ? D3D12_FENCE_FLAG_SHARED : D3D12_FENCE_FLAG_NONE,
+            &IID_ID3D12Fence, (void**)&drain_fence);
+    ok(SUCCEEDED(hr), "Failed to create fence, hr #%x.\n", hr);
+
+    if (FAILED(hr))
+    {
+        skip("Failed to create fence, skipping test ...\n");
+        ID3D12Fence_Release(signal_fence);
+        ID3D12Fence_Release(wait_fence);
+        destroy_test_context(&context);
+        return;
+    }
+
+#ifdef _WIN32
+    if (shared_handles)
+    {
+        hr = ID3D12Device_CreateSharedHandle(context.device, (ID3D12DeviceChild*)signal_fence,
+                NULL, GENERIC_ALL, NULL, &shared_signal);
+        ok(SUCCEEDED(hr), "Failed to create shared handle, hr #%x.\n", hr);
+        hr = ID3D12Device_CreateSharedHandle(context.device, (ID3D12DeviceChild*)wait_fence,
+                NULL, GENERIC_ALL, NULL, &shared_wait);
+        ok(SUCCEEDED(hr), "Failed to create shared handle, hr #%x.\n", hr);
+        hr = ID3D12Device_CreateSharedHandle(context.device, (ID3D12DeviceChild*)drain_fence,
+                NULL, GENERIC_ALL, NULL, &shared_drain);
+        ok(SUCCEEDED(hr), "Failed to create shared handle, hr #%x.\n", hr);
+
+        ID3D12Fence_Release(signal_fence);
+        ID3D12Fence_Release(wait_fence);
+        ID3D12Fence_Release(drain_fence);
+
+        hr = ID3D12Device_OpenSharedHandle(context.device, shared_signal, &IID_ID3D12Fence, (void**)&signal_fence);
+        ok(SUCCEEDED(hr), "Failed to open shared handle, hr #%x.\n", hr);
+        hr = ID3D12Device_OpenSharedHandle(context.device, shared_wait, &IID_ID3D12Fence, (void**)&wait_fence);
+        ok(SUCCEEDED(hr), "Failed to open shared handle, hr #%x.\n", hr);
+        hr = ID3D12Device_OpenSharedHandle(context.device, shared_drain, &IID_ID3D12Fence, (void**)&drain_fence);
+        ok(SUCCEEDED(hr), "Failed to open shared handle, hr #%x.\n", hr);
+
+        /* OpenSharedHandle takes a kernel level reference on the HANDLE. */
+        hr = ID3D12Device_OpenSharedHandle(context.device, shared_signal, &IID_ID3D12Fence, (void**)&signal_fence_dup);
+        ok(SUCCEEDED(hr), "Failed to open shared handle, hr #%x.\n", hr);
+        hr = ID3D12Device_OpenSharedHandle(context.device, shared_wait, &IID_ID3D12Fence, (void**)&wait_fence_dup);
+        ok(SUCCEEDED(hr), "Failed to open shared handle, hr #%x.\n", hr);
+        hr = ID3D12Device_OpenSharedHandle(context.device, shared_drain, &IID_ID3D12Fence, (void**)&drain_fence_dup);
+        ok(SUCCEEDED(hr), "Failed to open shared handle, hr #%x.\n", hr);
+
+        /* Observed behavior: Closing the last reference to the kernel HANDLE object unblocks all waiters.
+         * This isn't really implementable in Wine as it stands since applications are free to share
+         * the HANDLE and Dupe it arbitrarily.
+         * For now, assume this is not a thing, we can report TDR-like situations if this comes up in practice. */
+        if (shared_signal)
+            CloseHandle(shared_signal);
+        if (shared_wait)
+            CloseHandle(shared_wait);
+        if (shared_drain)
+            CloseHandle(shared_drain);
+    }
+#endif
+
+    memset(&queue_desc, 0, sizeof(queue_desc));
+    queue_desc.Flags = D3D12_COMMAND_QUEUE_FLAG_NONE;
+    queue_desc.Priority = D3D12_COMMAND_QUEUE_PRIORITY_NORMAL;
+    queue_desc.Type = D3D12_COMMAND_LIST_TYPE_COMPUTE;
+
+    src = create_default_buffer(context.device, 256 * 1024 * 1024, D3D12_RESOURCE_FLAG_NONE, D3D12_RESOURCE_STATE_COPY_SOURCE);
+    dst = create_default_buffer(context.device, 256 * 1024 * 1024, D3D12_RESOURCE_FLAG_NONE, D3D12_RESOURCE_STATE_COPY_DEST);
+
+    ID3D12Device_CreateCommandQueue(context.device, &queue_desc, &IID_ID3D12CommandQueue, (void**)&compute_queue);
+
+    for (i = 0; i < 2; i++)
+    {
+        ID3D12Device_CreateCommandAllocator(context.device, D3D12_COMMAND_LIST_TYPE_COMPUTE,
+                &IID_ID3D12CommandAllocator, (void**)&allocator[i]);
+        ID3D12Device_CreateCommandList(context.device, 0, D3D12_COMMAND_LIST_TYPE_COMPUTE, allocator[i], NULL,
+                &IID_ID3D12GraphicsCommandList, (void**)&list[i]);
+    }
+
+    /* Heavy copy action. */
+    for (i = 0; i < 128; i++)
+    {
+        ID3D12GraphicsCommandList_CopyResource(list[0], dst, src);
+        ID3D12GraphicsCommandList_CopyResource(list[1], src, dst);
+    }
+
+    ID3D12GraphicsCommandList_Close(list[0]);
+    ID3D12GraphicsCommandList_Close(list[1]);
+
+    /* Note on ref-count checks: The debug layers can take transient public ref-counts it seems. */
+
+    ID3D12CommandQueue_ExecuteCommandLists(context.queue, 1, (ID3D12CommandList * const *)&list[0]);
+    ID3D12CommandQueue_Signal(context.queue, signal_fence, 1);
+    /* Validate that signal/wait does not take public ref-counts. */
+    value = get_refcount(signal_fence);
+    ok(value == 1, "Unexpected ref-count %u\n", value);
+
+    /* The GPU copy is 32 GB worth of BW. There is literally zero chance it would have completed in this amount of time. */
+    value = (UINT)ID3D12Fence_GetCompletedValue(signal_fence);
+    ok(value == 0, "Unexpected signal event %u.\n", value);
+
+    /* Try waiting for a signal that never comes. We'll be able to unblock this wait
+     * when we fully release the fence. */
+    ID3D12CommandQueue_Wait(compute_queue, signal_fence, UINT64_MAX);
+    value = get_refcount(signal_fence);
+    ok(value == 1, "Unexpected ref-count %u\n", value);
+
+    ID3D12CommandQueue_Signal(compute_queue, wait_fence, 1);
+    value = get_refcount(wait_fence);
+    ok(value == 1, "Unexpected ref-count %u\n", value);
+
+    /* The GPU copy is 32 GB worth of BW. There is literally zero chance it would have completed in this amount of time. */
+    value = (UINT)ID3D12Fence_GetCompletedValue(wait_fence);
+    ok(value == 0, "Unexpected signal event %u.\n", value);
+    value = (UINT)ID3D12Fence_GetCompletedValue(signal_fence);
+    ok(value == 0, "Unexpected signal event %u.\n", value);
+
+    ID3D12CommandQueue_Wait(compute_queue, wait_fence, 1);
+    value = get_refcount(wait_fence);
+    ok(value == 1, "Unexpected ref-count %u\n", value);
+
+    /* Check that we can queue up event completion.
+     * Again, verify that releasing the fence unblocks all waiters ... */
+    event = create_event();
+    ID3D12Fence_SetEventOnCompletion(signal_fence, UINT64_MAX, event);
+
+    if (signal_fence_dup)
+        ID3D12Fence_Release(signal_fence_dup);
+    if (wait_fence_dup)
+        ID3D12Fence_Release(wait_fence_dup);
+
+    /* The GPU copy is 32 GB worth of BW. There is literally zero chance it would have completed in this amount of time.
+     * Makes sure that the fences aren't signalled when we try to free them.
+     * (Sure, there is a theoretical race condition if GPU completes between this check and the release, but seriously ...). */
+    value = (UINT)ID3D12Fence_GetCompletedValue(signal_fence);
+    ok(value == 0, "Unexpected signal event %u.\n", value);
+    value = (UINT)ID3D12Fence_GetCompletedValue(wait_fence);
+    ok(value == 0, "Unexpected signal event %u.\n", value);
+
+    /* Test that it's valid to release fence while it's in flight.
+     * If we don't cause device lost and drain_fence is waited on successfully we pass the test. */
+    value = ID3D12Fence_Release(signal_fence);
+    ok(value == 0, "Unexpected fence ref-count %u.\n", value);
+    value = ID3D12Fence_Release(wait_fence);
+    ok(value == 0, "Unexpected fence ref-count %u.\n", value);
+
+    ID3D12CommandQueue_ExecuteCommandLists(compute_queue, 1, (ID3D12CommandList * const *)&list[1]);
+    ID3D12CommandQueue_Signal(compute_queue, drain_fence, 1);
+
+    wait_event(event, INFINITE);
+    destroy_event(event);
+    ID3D12Fence_SetEventOnCompletion(drain_fence, 1, NULL);
+    value = (UINT)ID3D12Fence_GetCompletedValue(drain_fence);
+    ok(value == 1, "Expected fence wait value 1, but got %u.\n", value);
+
+    if (drain_fence_dup)
+    {
+        /* Check we observe the counter in sibling fences as well. */
+        value = (UINT)ID3D12Fence_GetCompletedValue(drain_fence_dup);
+        ok(value == 1, "Expected fence wait value 1, but got %u.\n", value);
+        ID3D12Fence_Release(drain_fence_dup);
+    }
+
+    value = ID3D12Fence_Release(drain_fence);
+    ok(value == 0, "Unexpected fence ref-count %u.\n", value);
+
+    /* Early freeing of fences might signal the drain fence too early, causing GPU hang. */
+    wait_queue_idle(context.device, context.queue);
+    wait_queue_idle(context.device, compute_queue);
+
+    ID3D12CommandQueue_Release(compute_queue);
+    for (i = 0; i < 2; i++)
+    {
+        ID3D12CommandAllocator_Release(allocator[i]);
+        ID3D12GraphicsCommandList_Release(list[i]);
+    }
+    ID3D12Resource_Release(dst);
+    ID3D12Resource_Release(src);
+
+    destroy_test_context(&context);
+}
+
+void test_fence_wait_robustness(void)
+{
+    test_fence_wait_robustness_inner(false);
+}
+
+void test_fence_wait_robustness_shared(void)
+{
+#ifdef _WIN32
+    test_fence_wait_robustness_inner(true);
+#else
+    skip("Shared fences not supported on native Linux build.\n");
+#endif
+}
+
+void test_fence_wait_multiple_inner(bool shared_handles)
+{
+    ID3D12CommandQueue *queue1, *queue2;
+    struct test_context context;
+    ID3D12Device1 *device1;
+    ID3D12Fence *fences[4];
+    UINT64 values[4];
+    unsigned int i;
+    HANDLE event;
+    HRESULT hr;
+    DWORD ret;
+
+    if (!init_compute_test_context(&context))
+        return;
+
+    if (ID3D12Device_QueryInterface(context.device, &IID_ID3D12Device1, (void**)&device1))
+    {
+        skip("Failed to query ID3D12Device1 interface, skipping test ...\n");
+        destroy_test_context(&context);
+        return;
+    }
+
+    memset(fences, 0, sizeof(fences));
+
+    for (i = 0; i < ARRAY_SIZE(fences); i++)
+    {
+        hr = ID3D12Device1_CreateFence(device1, 0,
+                shared_handles ? D3D12_FENCE_FLAG_SHARED : D3D12_FENCE_FLAG_NONE,
+                &IID_ID3D12Fence, (void**)&fences[i]);
+        todo_if(shared_handles) ok(SUCCEEDED(hr), "Failed to create fence, hr #%x.\n", hr);
+
+        if (FAILED(hr))
+            break;
+    }
+
+    if (FAILED(hr))
+    {
+        skip("Failed to create fence, skipping test ...\n");
+
+        for (i = 0; i < ARRAY_SIZE(fences); i++)
+        {
+            if (fences[i])
+                ID3D12Fence_Release(fences[i]);
+        }
+
+        ID3D12Device1_Release(device1);
+        destroy_test_context(&context);
+        return;
+    }
+
+    queue1 = create_command_queue(context.device, D3D12_COMMAND_LIST_TYPE_DIRECT, D3D12_COMMAND_QUEUE_PRIORITY_NORMAL);
+    queue2 = create_command_queue(context.device, D3D12_COMMAND_LIST_TYPE_COMPUTE, D3D12_COMMAND_QUEUE_PRIORITY_NORMAL);
+
+    event = create_event();
+    ret = wait_event(event, 0);
+    ok(ret == WAIT_TIMEOUT, "Got unexpected return value %u.\n", ret);
+
+    /* Test various invalid parameter combinations that should not crash */
+    hr = ID3D12Device1_SetEventOnMultipleFenceCompletion(device1,
+            NULL, NULL, 0, D3D12_MULTIPLE_FENCE_WAIT_FLAG_ALL, NULL);
+    ok(hr == E_INVALIDARG, "Got unexpected result, hr %#x.\n", hr);
+    hr = ID3D12Device1_SetEventOnMultipleFenceCompletion(device1,
+            fences, values, 0, D3D12_MULTIPLE_FENCE_WAIT_FLAG_ANY, event);
+    ok(hr == E_INVALIDARG, "Got unexpected result, hr %#x.\n", hr);
+    hr = ID3D12Device1_SetEventOnMultipleFenceCompletion(device1,
+            fences, values, 4, 0xFFFFFFFF, event);
+    ok(hr == E_INVALIDARG, "Got unexpected result, hr %#x.\n", hr);
+
+    /* The D3D12 runtime will crash here
+    hr = ID3D12Device1_SetEventOnMultipleFenceCompletion(device1,
+            fences, NULL, 1, D3D12_MULTIPLE_FENCE_WAIT_FLAG_ALL, event);
+    ok(hr == E_INVALIDARG, "Got unexpected result, hr %#x.\n", hr);
+    */
+
+    /* Test simple single-wait scenario */
+    values[0] = 1;
+
+    hr = ID3D12Device1_SetEventOnMultipleFenceCompletion(device1,
+            fences, values, 1, D3D12_MULTIPLE_FENCE_WAIT_FLAG_ALL, event);
+    ok(hr == S_OK, "Failed to set event on multiple fence completion, hr %#x.\n", hr);
+    ret = wait_event(event, 0);
+    ok(ret == WAIT_TIMEOUT, "Got unexpected return value %u.\n", ret);
+
+    ID3D12Fence_Signal(fences[0], 1);
+    ret = wait_event(event, INFINITE);
+    ok(ret == WAIT_OBJECT_0, "Got unexpected return value %u.\n", ret);
+    ret = wait_event(event, 0);
+    ok(ret == WAIT_TIMEOUT, "Got unexpected return value %u.\n", ret);
+
+    /* Test complex multi-wait scenario with WAIT_ALL */
+    values[0] = 2;
+    values[1] = 1;
+    values[2] = 53;
+    values[3] = 11;
+
+    hr = ID3D12Device1_SetEventOnMultipleFenceCompletion(device1,
+            fences, values, 4, D3D12_MULTIPLE_FENCE_WAIT_FLAG_ALL, event);
+    ok(hr == S_OK, "Failed to set event on multiple fence completion, hr %#x.\n", hr);
+    ret = wait_event(event, 0);
+    ok(ret == WAIT_TIMEOUT, "Got unexpected return value %u.\n", ret);
+
+    ID3D12Fence_Signal(fences[1], 4);
+
+    ret = wait_event(event, 100);
+    ok(ret == WAIT_TIMEOUT, "Got unexpected return value %u.\n", ret);
+
+    queue_signal(queue1, fences[0], 2);
+    queue_signal(queue2, fences[3], 14);
+    ID3D12Fence_Signal(fences[2], 53);
+
+    ret = wait_event(event, INFINITE);
+    ok(ret == WAIT_OBJECT_0, "Got unexpected return value %u.\n", ret);
+    ret = wait_event(event, 0);
+    ok(ret == WAIT_TIMEOUT, "Got unexpected return value %u.\n", ret);
+
+    hr = ID3D12Device1_SetEventOnMultipleFenceCompletion(device1,
+            fences, values, 4, D3D12_MULTIPLE_FENCE_WAIT_FLAG_ALL, NULL);
+    ok(hr == S_OK, "Failed to set event on multiple fence completion, hr %#x.\n", hr);
+
+    /* Test complex multi-wait with WAIT_ANY */
+    values[0] = 100;
+    values[1] = 200;
+    values[2] = 300;
+    values[3] = 400;
+
+    hr = ID3D12Device1_SetEventOnMultipleFenceCompletion(device1,
+            fences, values, 4, D3D12_MULTIPLE_FENCE_WAIT_FLAG_ANY, event);
+    ok(hr == S_OK, "Failed to set event on multiple fence completion, hr %#x.\n", hr);
+    ret = wait_event(event, 0);
+    ok(ret == WAIT_TIMEOUT, "Got unexpected return value %u.\n", ret);
+
+    queue_signal(queue1, fences[1], 200);
+    queue_signal(queue2, fences[2], 300);
+    ret = wait_event(event, INFINITE);
+    ok(ret == WAIT_OBJECT_0, "Got unexpected return value %u.\n", ret);
+
+    ID3D12Fence_Signal(fences[0], 100);
+
+    ret = wait_event(event, 100);
+    ok(ret == WAIT_TIMEOUT, "Got unexpected return value %u.\n", ret);
+
+    hr = ID3D12Device1_SetEventOnMultipleFenceCompletion(device1,
+            fences, values, 4, D3D12_MULTIPLE_FENCE_WAIT_FLAG_ANY, NULL);
+    ok(hr == S_OK, "Failed to set event on multiple fence completion, hr %#x.\n", hr);
+
+    hr = ID3D12Device1_SetEventOnMultipleFenceCompletion(device1,
+            fences, values, 4, D3D12_MULTIPLE_FENCE_WAIT_FLAG_ANY, event);
+    ok(hr == S_OK, "Failed to set event on multiple fence completion, hr %#x.\n", hr);
+    ret = wait_event(event, INFINITE);
+    ok(ret == WAIT_OBJECT_0, "Got unexpected return value %u.\n", ret);
+
+    for (i = 0; i < ARRAY_SIZE(fences); i++)
+        ID3D12Fence_Release(fences[i]);
+
+    destroy_event(event);
+
+    ID3D12CommandQueue_Release(queue1);
+    ID3D12CommandQueue_Release(queue2);
+    ID3D12Device1_Release(device1);
+    destroy_test_context(&context);
+}
+
+void test_fence_wait_multiple(void)
+{
+    test_fence_wait_multiple_inner(false);
+}
+
+void test_fence_wait_multiple_shared(void)
+{
+#ifdef _WIN32
+    test_fence_wait_multiple_inner(true);
+#else
+    skip("Shared fences not supported on native Linux build.\n");
+#endif
+}
+
+static void test_concurrent_signal_stress_inner(void)
+{
+    ID3D12CommandQueue *queue1, *queue2;
+    struct test_context context;
+    ID3D12Fence *fences[4];
+    unsigned int iteration;
+    UINT64 fence_values[4];
+    ID3D12Device1 *device1;
+    bool wait_success;
+    unsigned int i;
+    HANDLE event;
+    HRESULT hr;
+
+    event = create_event();
+
+    if (!init_compute_test_context(&context))
+        return;
+
+    ID3D12Device_QueryInterface(context.device, &IID_ID3D12Device1, (void**)&device1);
+    queue1 = create_command_queue(context.device, D3D12_COMMAND_LIST_TYPE_DIRECT, D3D12_COMMAND_QUEUE_PRIORITY_NORMAL);
+    queue2 = create_command_queue(context.device, D3D12_COMMAND_LIST_TYPE_COMPUTE, D3D12_COMMAND_QUEUE_PRIORITY_NORMAL);
+
+    for (i = 0; i < ARRAY_SIZE(fences); i++)
+    {
+        hr = ID3D12Device_CreateFence(context.device, 0, D3D12_FENCE_FLAG_NONE, &IID_ID3D12Fence, (void**)&fences[i]);
+        ok(SUCCEEDED(hr), "Failed to create fence, hr #%x.\n", hr);
+        fence_values[i] = 0;
+    }
+
+    for (iteration = 0; iteration < 10; iteration++)
+    {
+        for (i = 0; i < 2; i++)
+        {
+            fence_values[0] += 3;
+            queue_signal(queue1, fences[0], fence_values[0]);
+            fence_values[0] += 4;
+            queue_signal(queue1, fences[0], fence_values[0]);
+            fence_values[1] += 5;
+            queue_signal(queue2, fences[1], fence_values[1]);
+            fence_values[2] += 6;
+            queue_signal(queue1, fences[2], fence_values[2]);
+            fence_values[3] += 7;
+            queue_signal(queue2, fences[3], fence_values[3]);
+        }
+
+        hr = ID3D12Device1_SetEventOnMultipleFenceCompletion(device1,
+                fences, fence_values, 4, D3D12_MULTIPLE_FENCE_WAIT_FLAG_ALL, event);
+        ok(SUCCEEDED(hr), "Failed to set event hr #%x.\n", hr);
+
+        wait_success = wait_event(event, 1000) == WAIT_OBJECT_0;
+
+        ok(wait_success, "Failed to wait for event. GPU likely hung.\n");
+        if (!wait_success)
+            goto err;
+    }
+
+    for (i = 0; i < ARRAY_SIZE(fences); i++)
+        wait_for_fence(fences[i], fence_values[i]);
+
+err:
+    ID3D12CommandQueue_Release(queue1);
+    ID3D12CommandQueue_Release(queue2);
+    for (i = 0; i < ARRAY_SIZE(fences); i++)
+        ID3D12Fence_Release(fences[i]);
+    ID3D12Device1_Release(device1);
+    destroy_event(event);
+    destroy_test_context(&context);
+}
+
+void test_concurrent_signal_stress(void)
+{
+    unsigned int i;
+    for (i = 0; i < 100; i++)
+    {
+        vkd3d_test_set_context("Test %u", i);
+        test_concurrent_signal_stress_inner();
+    }
+    vkd3d_test_set_context(NULL);
+}
