@@ -192,6 +192,7 @@ decl_test(test_combined_clip_and_cull_distances_dxbc);
 decl_test(test_combined_clip_and_cull_distances_dxil);
 decl_test(test_resource_allocation_info);
 decl_test(test_suballocate_small_textures);
+decl_test(test_suballocate_small_textures_size);
 decl_test(test_command_list_initial_pipeline_state);
 decl_test(test_blend_factor);
 decl_test(test_dual_source_blending_dxbc);
@@ -199,6 +200,7 @@ decl_test(test_dual_source_blending_dxil);
 decl_test(test_multisample_rendering);
 decl_test(test_multisample_resolve);
 decl_test(test_multisample_resolve_formats);
+decl_test(test_multisample_resolve_strongly_typed);
 decl_test(test_sample_mask_dxbc);
 decl_test(test_sample_mask_dxil);
 decl_test(test_coverage_dxbc);
@@ -271,6 +273,8 @@ decl_test(test_virtual_queries);
 decl_test(test_vrs);
 decl_test(test_vrs_dxil);
 decl_test(test_vrs_image);
+decl_test(test_vrs_depth_write_dxbc);
+decl_test(test_vrs_depth_write_dxil);
 decl_test(test_stress_suballocation);
 decl_test(test_stress_suballocation_multithread);
 decl_test(test_stress_suballocation_rebar);
@@ -343,6 +347,7 @@ decl_test(test_fence_wait_multiple);
 decl_test(test_fence_wait_multiple_shared);
 decl_test(test_root_signature_empty_blob);
 decl_test(test_sparse_buffer_memory_lifetime);
+decl_test(test_sparse_depth_stencil_rendering);
 decl_test(test_rendering_no_attachments_layers);
 decl_test(test_memory_model_uav_coherence_thread_group_dxbc);
 decl_test(test_memory_model_uav_coherence_thread_group_dxil);
@@ -401,5 +406,19 @@ decl_test(test_renderpass_validation);
 decl_test(test_renderpass_rendering);
 decl_test(test_sampler_rounding);
 decl_test(test_scissor_clamping);
+decl_test(test_mismatching_rtv_dsv_size);
 decl_test(test_view_instancing);
 decl_test(test_shader_io_mismatch);
+decl_test(test_workgraph_basic);
+decl_test(test_workgraph_broadcast_input);
+decl_test(test_workgraph_thread_input);
+decl_test(test_workgraph_coalesced_input);
+decl_test(test_workgraph_two_level_broadcast);
+decl_test(test_workgraph_two_level_empty);
+decl_test(test_workgraph_basic_recursion);
+decl_test(test_workgraph_cross_group_sharing);
+decl_test(test_workgraph_shared_inputs);
+decl_test(test_workgraph_local_root_signature);
+decl_test(test_sm68_draw_parameters);
+decl_test(test_sm68_wave_size_range);
+decl_test(test_sm68_sample_cmp_bias_grad);
